c**********************************************************************
c
c                       ===============
c                       PROGRAM : PRUNE
c                       ===============
c
c       This program reads in the basis set truncation file <fileid>.bss
c       written by EPRBF, lists the data on the screen, and prunes the
c       basis set in the index file according to the ptol user specified.
c
c       Includes:
c               stddim.inc
c               stdio.inc
c               eprdat.inc
c               indexf.inc
c               fnames.inc
c               rndoff.inc
c
c       Uses:
c               getids.f
c               setnam.f
c               prmpt.f
c               rdfdat.f
c               wrfdat.f
c
c       modified by SHL 2-DEC-92  from TNLF for pruning
c
c**********************************************************************
c
      program prune
c
      include 'stddim.inc'
      include 'stdio.inc'
      include 'eprdat.inc'
      include 'indexf.inc'
      include 'fnames.inc'
      include 'rndoff.inc'
c
      logical fexist,fprmpt
      character*80 line
      integer i,j,maxle,maxlo,maxk,maxm,maxpi,maxlkm,
     #     minle,minlo,mink,minm,minpi,n,nfiles,nbas
      double precision basis(2,mxdim),bmax,pcut,ftoken
c
      logical prmpt
      external prmpt,ftoken
c
c######################################################################
c
      write(luttyo,1000)
      write(luttyo,1010)
c
      call getids(nfiles,1)
      if (nfiles.eq.0) stop
      fileid=files(1)
c
      ips=idxps(1)
      call setnam
c
c----------------------------------------------------------------------
c     read in parameters from parameter file
c----------------------------------------------------------------------
c
      inquire(file=prnamo,exist=fexist)
c
      if (fexist) then 
        call rdfdat(prnamo,ierr)
      else
	write (luttyo,1100) prnamo(:namlth)
        stop
      endif
c
      open (unit=ludisk,file=bsname,status='old',
     #     access='sequential',form='unformatted')
      read (ludisk) (basis(1,i),basis(2,i),i=1,ndim)
      close (unit=ludisk)
c
c----------------------------------------------------------------------
c     normalize to the largest value
c----------------------------------------------------------------------
c
      bmax=basis(2,1)
      do 10 i=2,ndim
        if (bmax.lt.basis(2,i)) bmax=basis(2,i)
 10   continue
c
      do 20 i=1,ndim
        basis(2,i)=basis(2,i)/bmax
 20   continue
c
c----------------------------------------------------------------------
c     Read basis set information from index file
c----------------------------------------------------------------------
c
      inquire(file=ixnamo,exist=fexist)
      if (fexist) then
        open (unit=ludisk,file=ixnamo,status='old',
     #        access='sequential',form='unformatted')
        read (ludisk) ndim
        read (ludisk) (pi1(i),qi1(i),l1(i),jk1(i),k1(i),
     #                 jm1(i),m1(i),i=1,ndim)
        close (unit=ludisk)
      else
        write (luttyo,1020)
        stop
      end if
c
c----------------------------------------------------------------------
c     Get the tolerance for pruning
c----------------------------------------------------------------------
c                                      
      pcut=ptol
      write (luttyo,1030) pcut
      fprmpt=prmpt(line)
      if (fprmpt) ptol=ftoken( line )
c
c----------------------------------------------------------------------
c     loop over basis set data
c----------------------------------------------------------------------
c                                      
      nbas=0
c
      maxle=-1
      minle=lemx+1
      maxlo=-1
      minlo=lomx+1
      maxk=-1
      mink=kmx+1
      maxm=-(mmx+1)
      minm=-maxm
      maxpi=-(ipnmx+1)
      minpi=ipnmx+1
      maxlkm=-1
c
      do 210 j=1,ndim
c
        if (basis(2,j).lt.ptol) go to 210
c
        n=l1(j)
        if (ipar(n).eq.1) then
          if (n.gt.maxle) maxle=n
          if (n.lt.minle) minle=n
        else
          if (n.gt.maxlo) maxlo=n
          if (n.lt.minlo) minlo=n
        end if
c
        n=k1(j)
        if (n.gt.maxk) maxk=n
        if (n.lt.mink) mink=n
c
        n=m1(j)
        if (n.gt.maxm) maxm=n
        if (n.lt.minm) minm=n
c
        n=pi1(j)
        if (n.gt.maxpi) maxpi=n
        if (n.lt.minpi) minpi=n
c
        n=l1(j)+k1(j)+iabs(m1(j))-lemx
        if (n.gt.maxlkm) maxlkm=n
c
        nbas=nbas+1
        pi1(nbas)=pi1(j)
        qi1(nbas)=qi1(j)
        l1(nbas)=l1(j)
        jk1(nbas)=jk1(j)
        k1(nbas)=k1(j)
        jm1(nbas)=jm1(j)
        m1(nbas)=m1(j)
c
 210  continue
c
      if (maxlo.eq.-1) maxlo=0
      if (minlo.eq.lomx+1) minlo=0
c
      write (luttyo,1050) ndim,nbas
      write(luttyo,1080)
      write(luttyo,1090) ptol,minle,maxle,minlo,maxlo,mink,maxk,
     #     minm,maxm,minpi,maxpi,maxlkm
c
      write (luttyo,1060)
c
      ndim=nbas
      lemx=maxle
      lomx=maxlo
      kmx=maxk
      mmx=maxm
      ipnmx=maxpi

      open (unit=ludisk,file=ixname,status='unknown',
     #      access='sequential',form='unformatted')
      write (ludisk) ndim
      write (ludisk) (pi1(i),qi1(i),l1(i),jk1(i),k1(i),
     #               jm1(i),m1(i),i=1,ndim)
      close (unit=ludisk)
c
      call wrfdat(prname)
c
c
c----------------------------------------------------------------------
c     exit from program
c----------------------------------------------------------------------
c
      write(luttyo,1000)
      stop
c
c======================================================================
c    format statements
c======================================================================
c
 1000 format(//,2x,70('#'),//)
 1010 format(28x,'PROGRAM : PRUNE',/,29x,'--------------',/)
 1020 format(/,10x,'!!! INDEX FILE DOES NOT EXIST !!!')
 1030 format(/,2x,'cut-off tolerance for pruning : ',g12.4)
 1050 format(/,2x,'old dimension = ',i4,5x,'new dimension = ',i5)
 1060 format(/,2x,'updating files : <id>.parx, <id>.indx')
 1080 format(//2x,70('-')/
     #       3x,'error',7x,'Le',8x,'Lo',8x,'K',9x,'M',9x,'pI',
     #       5x,'LKMdff'/,12x,5('min  max  '),'max'/ )
 1090 format(g10.3,11(i5))
 1100 format(/,20x,'*** CANNOT FIND FILE ',a,' ***',/)
c
c======================================================================
c
      end
