c----------------------------------------------------------------------
c                    =====================
c                     subroutine SETFLG
c                    =====================
c
c      This subroutine interprets <flag> string and sets various flags
c      The flags active are :
c
c		-d : reset to the default flags, which calculates the
c		     SLE matrix elements and starting vector elements
c		     and does not store these into a file.
c		-r : record the detailed information
c		-m : store the matrix elements into file with extension
c		     <.mtx_>.
c		-v : store the vector elements into file with extension
c		     <.stv_>.
c		-t : store the tridiagonal matrix into file with extension
c		     <.tdl_>.
c		-n : do not calculate eigenvector in EPRLF/CGF calculation.
c		-p : perform plain Lanczos or CG eigenvalue calculation.
c		     no check for the spurious and duplicate eigenvalues
c		-e : reads the eigenvalues from file <.egv_>.
c               -l : uses a single starting vector for diagonal subspace
c                    in EPRESA calculation.
c
c  Includes
c     stddim.inc   Definition of mxcalc
c     eprdat.inc   Definition of ips
c     fnames.inc   Definition of names in common /fnames/
c
c----------------------------------------------------------------------
      subroutine setflg(flgstr)
c
      implicit none
c
      include 'stddim.inc'
      include 'eprdat.inc'
      include 'fnames.inc'
c
      integer i
      character*5 flgstr
      character*1 chr
c
c  set the default flags
c
      recflg=.false.
      stmflg=.false.
      stvflg=ips.eq.1
      tdlflg=.false.
      evalflg=.false.
      evecflg=.true.
      tstflg=.true.
      lstflg=.false.
c
      do 10 i=1,5
      chr=flgstr(i:i)
      if ((chr.eq.'d').or.(chr.eq.'D').or.(chr.eq.' ')) return
      if ((chr.eq.'r').or.(chr.eq.'R')) recflg=.true.
      if ((chr.eq.'m').or.(chr.eq.'M')) stmflg=.true.
      if ((chr.eq.'v').or.(chr.eq.'V')) stvflg=.true.
      if ((chr.eq.'t').or.(chr.eq.'T')) tdlflg=.true.
      if ((chr.eq.'n').or.(chr.eq.'N')) evecflg=.false.
      if ((chr.eq.'p').or.(chr.eq.'P')) tstflg=.false.
      if ((chr.eq.'e').or.(chr.eq.'E')) evalflg=.true.
      if ((chr.eq.'l').or.(chr.eq.'L')) lstflg=.true.
 10   continue
c
      return
      end
