c**********************************************************************
c
c                   =================
c                   SUBROUTINE : CSLN
c                   =================
c
c   This subroutine evaluates the final ritz eigenvector of a complex
c   symmetric matrix for a given T eignevector, using Lanczos algorithm.
c
c	Arguments:
c               x	initial lanczos vector
c		y	work space
c		nstep	dimension of T matrix
c		ndim	dimension of R matrix
c		vt	T vector
c		vr	ritz vector
c
c       Includes:
c               stddim.inc
c
c       Uses:
c               scmvm.f
c               zaxpy.f
c               znormu.f
c               zdotu.f
c               zscsw.f
c
c**********************************************************************
c
        subroutine csln(x,y,nstep,ndim,vt,vr)
c
        include 'stddim.inc'
c
        integer nstep,ndim
        complex*16 x,y,a,b,vt,vr
        dimension x(mxdim),y(mxdim)
        dimension vt(mxstep),vr(mxdim)
c
        integer nl,i
        complex*16 u
        dimension u(mxdim)
c
        complex*16 zdotu,znormu
        external zdotu,znormu
c
c######################################################################
c
c=======================================================================
c               loop over Lanczos steps
c=======================================================================
c
        do 100 nl=1,nstep
c
c----------------------------------------------------------------------
c               calculate eigenvector
c----------------------------------------------------------------------
c
         do 10 i=1,ndim
            vr(i)=vr(i)+x(i)*vt(nl)
10       continue
c
c----------------------------------------------------------------------
c             update y by doing matrix-vector multiply then adding
c             old y vector to the result, i.e.,
c                       y <- A*x+y = u+y
c----------------------------------------------------------------------
c
         call scmvm(x,u,ndim)
         call zaxpy(u,y,dcmplx(1.0D0,0.0D0),ndim)
c
c----------------------------------------------------------------------
c             calculate alpha
c----------------------------------------------------------------------
c
         a=zdotu(x,y,ndim)
c
c----------------------------------------------------------------------
c             update y
c----------------------------------------------------------------------
c
         call zaxpy(x,y,-a,ndim)
c
c----------------------------------------------------------------------
c             calculate beta
c----------------------------------------------------------------------
c
         b=znormu(y,ndim)
c
c----------------------------------------------------------------------
c             rearrange vectors
c----------------------------------------------------------------------
c
         call zscsw(y,x,b,ndim)
c
c=======================================================================
c             end of loop over Lanczos steps
c=======================================================================
c
100     continue
c
c----------------------------------------------------------------------
c             return to calling program
c----------------------------------------------------------------------
c
        return
        end
