c*********************************************************************
c
c                       ================
c                       PROGRAM : CWSPEC
c                       ================
c
c	This program reads the eigenvalues and weighting factors
c       obtained from EPRESA or EPREGV routines and calculates the
c       cw EPR spectrum according to the user-specified threshold
c       for the weighting factors.  This is meant to set up an
c       empirical relationship between the motional rate and the
c       wtol defined in LBLF and PCHECK routines.
c
c       Includes :
c               stddim.inc
c               stdio.inc
c               eprdat.inc
c               fnames.inc
c
c       Uses :
c               getids.f
c               setnam.f
c               rdfdat.f
c
c*********************************************************************
c
      program cwspec
c
      implicit none
c
      include 'stddim.inc'
      include 'stdio.inc'
      include 'eprdat.inc'
      include 'fnames.inc'
c
      logical fexist 
c
      integer i,j,k,n,ierr,ipsold
      double precision fld,dfield
      complex*16 czero,ci,eval,wt,spec
      parameter (czero=(0.D0,0.D0),ci=(0.D0,1.D0))
      dimension eval(xdimg),wt(xdimg),spec(mxnpt)
c
c#####################################################################
c
      call getids(n,1)
c
c---------------------------------------------------------------------
c     construct file names
c---------------------------------------------------------------------
c
      fileid = files(1)
      ips=idxps(1)
      call setnam
c
c----------------------------------------------------------------------
c     read in parameters from parameter file
c----------------------------------------------------------------------
c
      inquire(file=prname,exist=fexist)
c
      ipsold=ips      
      if (fexist) then
        call rdfdat(prname,ierr)
        ips=ipsold
      else
        write (luttyo,1100) prname(:namlth)
        write (lulog,1100) prname(:namlth)
        stop
      endif
c
      if (ierr.eq.1) then
        write (luttyo,1110) prname(:namlth)
        write (lulog,1110) prname(:namlth)
        stop
      else if (ierr.ne.0) then
        write (luttyo,1120) prname(:namlth)
        write (lulog,1120) prname(:namlth)
        stop
      end if
c
c----------------------------------------------------------------------
c     check to make sure that Exact Eigenvalues are desired
c----------------------------------------------------------------------
c
      write (luttyo,1010)
      read (luttyi,*) wtol
c
      if (itype.ne.5) then
        write (luttyo,5100) 
      end if
c
c---------------------------------------------------------------------
c     read the eigenvalues from eigenvalue file
c---------------------------------------------------------------------
c
      open (unit=ludisk,file=egname,status='unknown',
     #     access='sequential',form='formatted')
      read (ludisk,2420) (j,k,eval(i),wt(i),i=1,ndim)
      close (unit=ludisk)
c
c---------------------------------------------------------------------
c     calculate cw spectrum for the off-diagonal space
c---------------------------------------------------------------------
c
      dfield=(fieldf-fieldi)/dble(nfield-1)
c
      nev=0
 305  nev=nev+1
      if (cdabs(wt(nev)).ge.wtol) go to 305
      write (luttyo,2280) nev,wtol
c
      do 310 i=1,mxnpt
 310    spec(i)=czero
c
      do 320 i=1,nfield
        fld=fieldi+dfloat(i-1)*dfield
        do 330 j=1,nev
          spec(i)=spec(i)+wt(j)/(eval(j)+shiftr+(b0+shifti+fld)*ci)
 330  continue
 320  continue
c
      open (unit=ludisk,file=spname,status='unknown',
     #     access='sequential',form='formatted')
c    frequency sweep
c      write (ludisk,3200) (b0+fieldi+dfloat(i-1)*dfield,
c     #                     spec(i),i=1,nfield)
c    field sweep
      write (ludisk,3200) (b0+fieldi+dfloat(i-1)*dfield,
     #                     spec(nfield-i+1),i=1,nfield)

      close (unit=ludisk)
c
      stop
c
c=====================================================================
c     format statements
c=====================================================================
c
 1010 format(/2x,'Enter the tolerance of the absolute weighting',
     #       ' factor : ',$)
 1100 format(/,20x,'*** CANNOT FIND FILE ',a,' ***',/)
 1110 format(/,20X,'*** FILE ',a,' NOT EPRLF ***',/)
 1120 format(/,20X,'*** ERROR READING FILE ',a,' ***',/)
 2280 format(/,2x,i4,' eigenvalues included with wtol = ',g10.3)
 2420 format(i4,2x,i4,4x,g14.7,1x,g14.7,2x,g14.7,1x,g14.7)
c
 3200 format(1x,3(g14.7,','))
 5100 format(/,20x,'*** DOUBLE CHECK sweep range, points, ',
     #        'broadening ***')
c
c=====================================================================
c
      end
