c*********************************************************************
c
c                       ===============
c                       PROGRAM : EPRBF
c                       ===============
c
c       This program does the EPR "field sweep" conjugate gradients
c       calculation to determine the minimum truncation scheme.
c
c       The programs tnlf and prune, which was used to get the pruned
c       basis set is included in this program.
c
c       The program combas, which combines basis index sets of several
c       director angles for MOMD calculation is also inclued.
c
c       written by DJS 24-OCT-87
c       modified by SHL 10-JAN-93 for EPRP calculation.
c
c       Includes :
c               stddim.inc
c               stdio.inc
c               eprmat.inc
c               eprdat.inc
c               fnames.inc
c               indexf.inc
c               timer.inc (UNIX-specific timing with dtime function)
c
c       Uses :
c               getids.f
c               setnam.f
c               rdfdat.f
c               wrfdat.f
c               wrfprm.f
c               matrxo.f
c               fbaso.f
c               stvecp.f
c               cspccg.f
c               zdotu.f
c               zaxpy.f
c               scmvm.f
c
c*********************************************************************
c
      program eprbf
c
      implicit none
c
      include 'stddim.inc'
      include 'stdio.inc'
      include 'eprdat.inc'
      include 'eprmat.inc'
      include 'fnames.inc'
      include 'indexf.inc'
      include 'timer.inc'  
c
      double precision zero,one,pi
      parameter (zero=0.0D0,one=1.0D0)
c
      character*5 flgstr
      logical fexist,pcflag
c
      integer i,j,n,ncalc,ierr,ndone,ntotal,ipsold
c
      double precision b,x,y,spect,basis,invrs
      dimension b(2,mxdim),x(2,mxdim),y(2,mxdim),
     #          spect(2,mxnpt),basis(2,mxdim),invrs(mxdim)
c
      double precision test,scale,terror,terrmx,error,field,
     #                 dfield,bshft,bmax,pcut,cspsi,cputmp
c
      integer maxle,maxlo,maxk,maxm,maxpi,minle,minlo,
     #     mink,minm,minpi,ipar,nbas,icut,crdim,indim,iort,
     #     picr,qicr,lcr,jkcr,kcr,jmcr,mcr
c
      integer jqe2(mxdim),pi2(mxdim),qi2(mxdim),l2(mxdim),
     #        jk2(mxdim),k2(mxdim),jm2(mxdim),m2(mxdim)
c
      complex*16 temp,zdotu
      external zdotu,ipar
c
c#####################################################################
c
      pi=4.0d0*datan(one)
c
c----------------------------------------------------------------------
c Timing call changed for compatibility with UNIX
c      call cputime(cpu0,ircode)
c----------------------------------------------------------------------
      cputot = dtime( tarray )
c
      write (luttyo,1000)
      write (luttyo,1010)
      write (luttyo,1000)
c
      call getids(n,mxcalc)
c
      ntotal=0
      terrmx=zero
c
c---------------------------------------------------------------------
c     construct file names
c---------------------------------------------------------------------
c
      fileid = files(1)
      ips=idxps(1)
c
      if (ips.eq.0) then
        write (luttyo,1020)
        go to 999
      end if
c
      call setnam
      flgstr = flags(1)
      call setflg(flgstr)
c
c----------------------------------------------------------------------
c     open log file and write message
c----------------------------------------------------------------------
c
      open (unit=lulog,file=rlname,status='unknown',
     #      access='sequential',form='formatted')
      rewind (unit=lulog)
c
      write (lulog,1000)
      write (luttyo,1030) prname(:namlth)
      write (lulog,1030) prname(:namlth)
      write (lulog,1000)
c
c----------------------------------------------------------------------
c     read parameters from file
c----------------------------------------------------------------------
c
      inquire(file=prname,exist=fexist)
c
      ipsold=ips
      if (fexist) then
        call rdfdat(prname,ierr)
        if (ipsold.ne.ips) ips=ipsold
      else
        write (luttyo,1040) prname(:namlth)
        write (lulog,1040) prname(:namlth)
        go to 999
      endif
c                              ** write parameters on log file
      call wrfprm(prname,namlth,lulog)
c
c----------------------------------------------------------------------
c     check if FS & MOMD calculation is desired
c----------------------------------------------------------------------
c
      if (itype.ne.2) then
        write (luttyo,1050) 
        write (lulog,1050) 
        go to 999
      end if
c
      if (nort.le.1) then
        ncalc=1
      else
        ncalc=nort
      end if
c
c======================================================================
c     loop over calculations
c======================================================================
c
      crdim=0
c
      if (nort.gt.1) then
        write (luttyo,2000)
        write (lulog,2000)
      end if
c
      do 100 iort=1,ncalc
c
        if (nort.gt.1) then
          cspsi=dfloat(iort-1)/dfloat(nort-1)
          psi=qarcos(qextd(cspsi))*180.0D0/pi
        end if
c
c---------------------------------------------------------------------
c     calculate the matrix elements
c---------------------------------------------------------------------
c
        call fbaso(ixname)
c
        if (ndim.gt.mxdim) then
          write (luttyo,1060)
          write (lulog,1060)
          go to 999
        end if          
c
        call matrxo(ierr)
c
        if (ierr.ne.0) then
          write (luttyo,1070) 
          write (lulog,1070)
          if (ierr.eq.1) then
            write (luttyo,1080)
            write (lulog,1080) 
          end if
          go to 999
        end if
c                                     ** store matrix **
        if (stmflg.and.(iort.eq.1)) then 
          write (luttyo,*) 'iort=',iort,'storing matrix'
          open (unit=ludisk,file=mtname,status='unknown',
     #         access='sequential',form='unformatted')
          write (ludisk) (zdiag(1,i),zdiag(2,i),i=1,ndim)
          write (ludisk) (jzmat(i),i=1,ndim+1) 
          write (ludisk) (izmat(i),zmat(i),i=1,nelim)
          write (ludisk) (kzmat(i),i=1,ndim+1)
          write (ludisk) (izmat(mxel-i+1),zmat(mxel-i+1),i=1,nelre)
          close (unit=ludisk)
        end if
c
c---------------------------------------------------------------------
c     starting vector calculation
c---------------------------------------------------------------------
c
        call stvecp(b)
c                                      **  write starting vector
        if (stvflg) then
          open (unit=ludisk,file=vtname,status='unknown',
     #          access='sequential',form='unformatted')
          write (ludisk) (b(1,i),b(2,i),i=1,ndim)
          close (unit=ludisk)
        end if
c
        cpu1 = dtime( tarray )
c
        if (nort.le.1) then
          write (luttyo,1200) ndim,neltot,nelv,cpu1
          write (lulog,1200) ndim,neltot,nelv,cpu1
        end if
c
c----------------------------------------------------------------------
c     zero out arrays
c----------------------------------------------------------------------
c
        do 110 j=1,mxdim
          basis(1,j)=zero
          basis(2,j)=zero
 110    continue
c
        do 120 j=1,mxnpt
          spect(1,j)=zero
          spect(2,j)=zero
 120    continue
c
c----------------------------------------------------------------------
c     calculate the inverse of the preconditioning matrix
c----------------------------------------------------------------------
c
        do 130 j=1,ndim
          invrs(j)=one/sqrt(zdiag(1,j)+shiftr)
 130    continue
c
c======================================================================
c     loop over field positions
c======================================================================
c
        dfield=(fieldf-fieldi)/dble(nfield-1)
c
c  precondition the matrix to avoid diagonally dominant structure
c
        if(ips.eq.1) then
          bshft=shifti+b0
        else if(ips.eq.-1) then
          bshft=shifti-b0
        else
          bshft=shifti
        end if
c
        if (nort.le.1) write(lulog,3000)
c
        do 200 i=1,nfield
c
c----------------------------------------------------------------------
c     do the preconditioned CG iteration
c----------------------------------------------------------------------
c
          do 210 j=1,ndim
	    x(1,j)=zero
	    x(2,j)=zero
  210     continue
c
          field=fieldi+(i-1)*dfield
          temp=dcmplx(shiftr,bshft-field)
c
          pcflag=.false.
c
          call cspccg(b,ndim,nstep,cgtol,temp,
     #         invrs,x,ndone,error,pcflag)
c
c---------------------------------------------------------------------
c     find the true error in the solution
c----------------------------------------------------------------------
c
          call scmvm(x,y,ndim)
          call zaxpy(x,y,temp,ndim)
c
	  terror=zero
          do 220 j=1,ndim
            y(1,j)=b(1,j)-y(1,j)
            terror=terror+y(1,j)*y(1,j)  
            y(2,j)=b(2,j)-y(2,j)
	    terror=terror+y(2,j)*y(2,j)
 220      continue
c
          terror=sqrt(terror)
c
	  if (terror.gt.terrmx) terrmx=terror
          ntotal=ntotal+abs(ndone)+1
c
c----------------------------------------------------------------------
c     write out message
c----------------------------------------------------------------------
c
          if ((ndone.gt.0).and.(nort.le.1))  write(lulog,3010) i,
     #                                       ndone,error,terror,cpu3
          if (ndone.le.0) then
            write(luttyo,3020) i,nstep,-ndone,cgtol,error,terror,cpu3
            write(lulog,3020) i,nstep,-ndone,cgtol,error,terror,cpu3
          end if
c
c----------------------------------------------------------------------
c     store spectral information
c----------------------------------------------------------------------
c
          temp=zdotu(b,x,ndim)
          spect(1,i)=dreal(temp)/pi
          spect(2,i)=dimag(temp)/pi
c
c----------------------------------------------------------------------
c     sift through the solution vector
c----------------------------------------------------------------------
c
          scale=1.0D0/abs(temp)
          do 230 j=1,ndim
            test=sqrt(x(1,j)*x(1,j)+x(2,j)*x(2,j))*scale
            if (test.gt.basis(2,j)) then
              basis(2,j)=test
              basis(1,j)=field
            end if
 230      continue
c
c======================================================================
c     end of loop over field positions
c======================================================================
c
 200    continue
c
        if (nort.le.1) then
c
          write(lulog,3030)
c                                 ***  write out basis set information    
c
          open (unit=ludisk,file=bsname,status='unknown',
     #          access='sequential',form='unformatted')
          rewind (unit=ludisk)
          write (ludisk) (basis(1,i),basis(2,i),i=1,ndim)
          close (unit=ludisk)
c                                 ***  write out spectrum
c
          open (unit=ludisk,file=spname,status='unknown',
     #          access='sequential',form='formatted')
          rewind (unit=ludisk)
          write (ludisk,3200) (b0+fieldi+(i-1)*dfield,spect(1,i),
     #         spect(2,i),i=1,nfield)
          close (unit=ludisk)
c                                 ***  write out old parameters
c
          call wrfdat(prnamo)
c                                 ***  write out old index basis
c
          open (unit=ludisk,file=ixnamo,status='unknown',
     #          access='sequential',form='unformatted')
          write (ludisk) ndim
          write (ludisk) (pi1(i),qi1(i),l1(i),jk1(i),k1(i),
     #                    jm1(i),m1(i),i=1,ndim)
          close (unit=ludisk)
c
          write(luttyo,3100) ntotal,terrmx
          write(lulog,3100) ntotal,terrmx
c
        end if
c
        cpu2 = dtime( tarray )
        if (nort.le.1) then
          write (luttyo,3110) cpu2
          write (lulog,3110) cpu2
        end if
c
c======================================================================
c     perform tnlf & prune calculation
c======================================================================
c
c----------------------------------------------------------------------
c     write out header for database file
c----------------------------------------------------------------------
c
        if (nort.le.1) then
          open (unit=ludisk1,file=tsname,status='unknown',
     #          access='sequential',form='formatted')
c
          write(ludisk1,4200)
          write(luttyo,4200)
          write(lulog,4200)
        end if
c
c----------------------------------------------------------------------
c     normalize to the largest value
c----------------------------------------------------------------------
c
        bmax=basis(2,1)
        do 300 i=2,ndim
          if (bmax.lt.basis(2,i)) bmax=basis(2,i)
 300    continue
c
        do 310 i=1,ndim
          basis(2,i)=basis(2,i)/bmax
 310    continue
c
c----------------------------------------------------------------------
c     loop over cutoff values
c----------------------------------------------------------------------
c                                      
        if (nort.le.1) then
          icut=1
        else
          icut=17
        end if
c
        do 350 i=icut,17
c
          maxle=-1
          minle=lemx+1
          maxlo=-1
          minlo=lomx+1
          maxk=-1
          mink=kmx+1
          maxm=-(mmx+1)
          minm=-maxm
          maxpi=-(ipnmx+1)
          minpi=ipnmx+1
c
          nbas=0
c
          if (i.le.10) then
            pcut=(11.0D0-dfloat(i))*1.0D-2
          else if (i.eq.11) then
            pcut=0.5D-2
          else if (i.eq.12) then
            pcut=1.0D-3
          else if (i.eq.13) then
            pcut=0.5D-3
          else if (i.eq.14) then
            pcut=1.0D-4
          else if (i.eq.15) then
            pcut=0.5D-4
          else if (i.eq.16) then
            pcut=1.0D-5
          else
	    pcut=ptol
          end if
c
c----------------------------------------------------------------------
c     loop over basis set data
c----------------------------------------------------------------------
c                                      
          do 360 j=1,ndim
c
            if (basis(2,j).lt.pcut) go to 360
c
            n=l1(j)
            if (ipar(n).eq.1) then
              if (n.gt.maxle) maxle=n
              if (n.lt.minle) minle=n
            else
              if (n.gt.maxlo) maxlo=n
              if (n.lt.minlo) minlo=n
            end if
c
            n=k1(j)
            if (n.gt.maxk) maxk=n
            if (n.lt.mink) mink=n
c
            n=m1(j)
            if (n.gt.maxm) maxm=n
            if (n.lt.minm) minm=n
c
            n=pi1(j)
            if (n.gt.maxpi) maxpi=n
            if (n.lt.minpi) minpi=n
c
            nbas=nbas+1
c
            if (i.eq.17) then
              basis(1,nbas)=basis(1,j)
              basis(2,nbas)=basis(2,j)
              pi1(nbas)=pi1(j)
              qi1(nbas)=qi1(j)
              l1(nbas)=l1(j)
              jk1(nbas)=jk1(j)
              k1(nbas)=k1(j)
              jm1(nbas)=jm1(j)
              m1(nbas)=m1(j)
            end if
c
 360      continue
c
          if (maxlo.eq.-1) maxlo=0
          if (minlo.eq.lomx+1) minlo=0
c
          if (nort.le.1) then
            pcut=pcut*1.0D2
            write(ludisk1,4210) pcut,minle,maxle,minlo,maxlo,
     #                    mink,maxk,minm,maxm,minpi,maxpi,nbas
            if (i.eq.17) write(luttyo,4210) pcut,minle,maxle,minlo,
     #              maxlo,mink,maxk,minm,maxm,minpi,maxpi,nbas
            if (i.eq.17) write(lulog,4210) pcut,minle,maxle,minlo,
     #              maxlo,mink,maxk,minm,maxm,minpi,maxpi,nbas
          end if
c
 350    continue
c
        if (nort.le.1) then
          close(unit=ludisk1)
          write (luttyo,4000) ndim,nbas
          write (lulog,4000) ndim,nbas
        end if
c
c======================================================================
c     combine the basis set for MOMD
c======================================================================
c
        indim=0
        if (iort.eq.1) then
c   			          *** copy first basis set
          crdim=0
          indim=nbas
          do 400 i=1,nbas
            pi2(i)=pi1(i)
            qi2(i)=qi1(i)
            l2(i)=l1(i)
            jk2(i)=jk1(i)
            k2(i)=k1(i)
            jm2(i)=jm1(i)
            m2(i)=m1(i)
 400       continue
        else
c           *** go through basis set to see if it has new basis
c
          do 410 i=1,nbas
            picr=pi1(i)
            qicr=qi1(i)
            lcr=l1(i)
            jkcr=jk1(i)
            kcr=k1(i)
            jmcr=jm1(i)
            mcr=m1(i)
c
            do 420 j=1,crdim
              if ((picr.eq.pi2(j)).and.(qicr.eq.qi2(j)).and.
     #            (lcr.eq.l2(j)).and.(jkcr.eq.jk2(j)).and.
     #            (kcr.eq.k2(j)).and.(jmcr.eq.jm2(j)).and.
     #            (mcr.eq.m2(j)))  go to 410
 420        continue
c
            indim=indim+1
            pi2(crdim+indim)=picr
            qi2(crdim+indim)=qicr
            l2(crdim+indim)=lcr
            jk2(crdim+indim)=jkcr
            k2(crdim+indim)=kcr
            jm2(crdim+indim)=jmcr
            m2(crdim+indim)=mcr
c
 410      continue
c
        end if
c
        crdim=crdim+indim
c
        cpu3 = dtime( tarray )
        cputmp=cpu1+cpu2+cpu3
        cputot = cputot+cputmp
c
        if (nort.gt.1) then
          write (luttyo,2010) iort,psi,neltot,nelv,ndim,nbas,
     #                        crdim,cputmp
          write (lulog,2010) iort,psi,neltot,nelv,ndim,nbas,
     #                        crdim,cputmp
        end if
c
 100  continue
c
c======================================================================
c     end of loop over calculations
c======================================================================
c
c----------------------------------------------------------------------
c     Get MTS for the pruned (and combined) basis set
c----------------------------------------------------------------------
c                                      
      if (nort.gt.1) then
        maxle=-1
        minle=1000
        maxlo=-1
        minlo=1000
        maxk=-1
        mink=1000
        maxm=-1000
        minm=1000
        maxpi=-1000
        minpi=1000
c
        do 430 j=1,crdim
c
          n=l2(j)
          if (ipar(n).eq.1) then
            if (n.gt.maxle) maxle=n
            if (n.lt.minle) minle=n
          else
            if (n.gt.maxlo) maxlo=n
            if (n.lt.minlo) minlo=n
          end if
c
          n=k2(j)
          if (n.gt.maxk) maxk=n
          if (n.lt.mink) mink=n
c
          n=m2(j)
          if (n.gt.maxm) maxm=n
          if (n.lt.minm) minm=n
c
          n=pi2(j)
          if (n.gt.maxpi) maxpi=n
          if (n.lt.minpi) minpi=n
c
 430    continue
c
        if (maxlo.eq.-1) maxlo=0
        if (minlo.eq.1000) minlo=0
c
        write(luttyo,4200)
        write(luttyo,4220) ptol,minle,maxle,minlo,maxlo,mink,maxk,
     #       minm,maxm,minpi,maxpi,crdim
c
        write(lulog,4200)
        write(lulog,4220) ptol,minle,maxle,minlo,maxlo,mink,maxk,
     #       minm,maxm,minpi,maxpi,crdim
      end if
c
      ndim=crdim
      lemx=maxle
      lomx=maxlo
      kmx=maxk
      mmx=maxm
      ipnmx=maxpi
c
c---------------------------------------------------------------------
c     update parameter file, index file
c---------------------------------------------------------------------
c
      write (luttyo,5000)
c
      call wrfdat(prname)
c     
      open (unit=ludisk,file=ixname,status='unknown',
     #      access='sequential',form='unformatted')
      write (ludisk) ndim
      write (ludisk) (pi2(i),qi2(i),l2(i),jk2(i),k2(i),
     #               jm2(i),m2(i),i=1,ndim)
      close (unit=ludisk)
c
      cpu4 = dtime( tarray )
      cputot = cputot+cpu4
c
      write (luttyo,5100) cputot
      write (lulog,5100) cputot
c
      write(luttyo,1000)
      write(lulog,1000)	
c
 999  close (unit=lulog)
c
c---------------------------------------------------------------------
c                  exit program
c---------------------------------------------------------------------
c
      stop
c
c=====================================================================
c
 1000 format(/,2x,70('#'),/)
 1010 format(25x,'PROGRAM : EPRBF',/,25x,'---------------')
 1020 format(/10x,'!!! EPRBF IS FOR THE OFF-DIAGONAL SPACE !!!')
c
 1030 format(25x,'file : ',a)
 1040 format(/,10x,'*** FILE ',a,' NOT FOUND ***',/)
 1050 format(//,10x,
     #     '*** NOT SWEPT-FIELD CG WANTED ***')
 1060 format(/,20x,'*** MATRIX DIMENSION TOO LARGE ***',/,
     #     4x,'Use program compiled with bigger dimension or reduce',
     #     ' the dimension.')
 1070 format(//,10x,
     #     '*** SERIOUS ERROR ENCOUNTERED IN MATRIX GENERATION ***')
 1080 format(/,20x,'*** TOO MANY MATRIX ELEMENTS ***')
c
 1200 format(/,2x,'Dimension of the matrix : ',i5,
     #       /,2x,'Total number of non-zero entries : ',i6,
     #       /,2x,'Number of non-zero elements in starting vector ',
     #       ': ',i5,/,2x,'CPU time (seconds) : ',f10.2)
c
 2000 format(/,2x,70('-'),/,4x,'i',3x,'angle',4x,'neltot',3x,
     #      'nelv',3x,'updim',3x,'pdim',3x,'totdim',4x,'cpu',
     #      /,2x,70('-'),/)  
 2010 format(2x,i3,3x,f5.2,3x,i7,3x,i3,4x,i4,3x,i4,5x,i4,3x,f6.2)
c
 3000 format(/,2x,'+--------+-----------+----------------+',
     #            '----------------+',
     #       /,2x,'| Step   | Number of |   Modulus of   |',
     #            '  L2 norm of    |',
     #       /,2x,'| number | CG steps  |   rectanorm    |',
     #            '  true residual |',
     #       /,2x,'|--------+-----------+----------------+',
     #            '----------------|')
 3010 format(  2x,'| ',i6,' |   ',i6,'  | ',g14.7,' | ',g14.7,' | ',
     #       f10.2)
 3020 format(//,10x,'*** CG CALCULATION ',i3,' DID NOT CONVERGE ***',
     #     /,10x,'*** MAXIMUM NUMBER OF CG STEPS  : ',i3,' ***',
     #     /,10x,'*** NUMBER OF CG STEPS EXECUTED : ',i3,' ***',
     #     /,10x,'*** MAXIMUM CG ERROR ALLOWED    : ',g14.7,' ***',
     #     /,10x,'*** FINAL CG ERROR              : ',g14.7,' ***',
     #     /,10x,'*** TRUE ERROR                  : ',g14.7,' ***',
     #     /,10x,'*** CPU TIME (SECONDS)          : ',g14.2,' ***')
 3030 format(  2x,'+--------+-----------+----------------+',
     #            '----------------+',/)
c
 3100 format(/,2x,'Total number of maxtrix-vector multiplies : ',i5,
     #       /,2x,'Maximum true error : ',g14.7)
 3110 format(2x,'Total CPU time for eprbf calc. (seconds) : ',f10.2)
 3200 format(1x,3(g14.7,','))
c
 4000 format(/,2x,'unpruned dimension = ',i4,5x,
     #     'pruned dimension = ',i5)
 4200 format(/,2x,70('-')/
     #       3x,'error',7x,'Le',8x,'Lo',8x,'K',9x,'M',9x,'pI',
     #       5x,'pruned'/,12x,5('min  max  '),' dim.'/ )
 4210 format(g10.3,10(i5),i6)
 4220 format(g10.3,10(i5),i6,/,2x,70('-'))
 5000 format(/,2x,'updating files : <id>.parx, <id>.bssx, <id>.indx')
 5100 format(/,2x,'Total CPU time (seconds) : ',f10.2)
c
c=====================================================================
c
      end
