c*********************************************************************
c
c                       ===============
c                       PROGRAM : FSESE
c                       ===============
c
c       This program calculates a field-swept 2D-ESR spectrum, based
c       on the expression Millhauser and Freed '84.  Lineshape along
c       f2 direction is assumed to be dominated by the Gaussian
c       inhomogeneous broadening as in that paper, and it is always
c       real.
c       One has option not to use the approximation
c             *tr
c            O   O = I
c       and the output type along f1 axis.
c
c       NUCLEAR MODULATION is not supported.
c
c       Includes :
c               tddim.inc
c               stdio.inc
c               tdmat.inc
c               tdexp.inc
c               timer.inc
c               tdfnam.inc
c
c       Uses :
c               getidt.f
c               tdsnam.f
c               rdfinp.f
c               wrfinp.f
c               wrfout.f
c               fft.f
c               switch.f
c
c*********************************************************************
c
      program fsese
c
      implicit none
c
      include 'tddim.inc'
      include 'stdio.inc'
      include 'tdmat.inc'
      include 'tdexp.inc'
      include 'timer.inc'
      include 'tdfnam.inc'
c
      integer i,j,k,l,m,n,mxcal,ncalc,it1,it2,ierr,idgm,iout
      double precision t1,t2,f1,f2,freq1,freq2,res1,res2,
     #                 cst,expnt,gconv,amin,amax
c
      double precision zero,one,two,mone
      parameter (zero=0.0D0,one=1.0D0,two=2.0D0,mone=1.0D-3)
      double precision pi,twopi,gtf,cfact,itwopi
      parameter (pi=3.1415926535897932384D0,gtf=2.802471512)
      complex*16 czero,cone,ci,wt,sum1,sum2
      parameter (czero=(0.0D0,0.0D0),cone=(1.0D0,0.0D0),
     #           ci=(0.0D0,1.0D0))
c
      logical fexist    
c
      character*30 xaxist,yaxist,xaxisf,yaxisf
      data xaxisf/'f2 (MHz)'/
      data yaxisf/'f1 (MHz)'/
      data xaxist/'t2 (ns)'/
      data yaxist/'t1 (ns)'/
c
      complex*16 egvalx(mxegv),egvaly(mxegv),egvalz(mxegv)
c
c#####################################################################
c
c----------------------------------------------------------------------
c Timing call changed for compatibility with UNIX
c      call cputime(cpu0,ircode)
c----------------------------------------------------------------------
c
      cputot = dtime( tarray )
      twopi=two*pi
      itwopi=one/twopi
      cfact=twopi*gtf
c
      write (luttyo,1000)
      write (luttyo,1010)
c
      mxcal=10
      call getidt(n,mxcal)
c
c======================================================================
c     loop over calculations
c======================================================================
c
      do 900 ncalc=1,n
c
c---------------------------------------------------------------------
c     construct file names
c---------------------------------------------------------------------
c
        ioflid = iofile(ncalc)
        dtflid = dtfile(ncalc)
        call tdsnam
c
c----------------------------------------------------------------------
c     open log file and write out header 
c----------------------------------------------------------------------
c
        open (unit=lulog,file=rpname,access='sequential',
     #        form='formatted')
c
        rewind (unit=lulog)
c
        write (luttyo,1000)
        write (luttyo,5000) ncalc,ipname(:namlth)
        write (lulog,5000) ncalc,ipname(:namlth)
c
c----------------------------------------------------------------------
c     read the parameters from parameter file
c----------------------------------------------------------------------
c
        inquire(file=ipname,exist=fexist)
c
        if (fexist) then
          call rdfinp(ipname)
        else
          write (luttyo,1100) ipname(:namlth)
          write (lulog,1100) ipname(:namlth)
          go to 999
        endif
c
        if ((itimed.eq.1).and.(idtf.ne.0)) then
          write (luttyo,1110)
          write (lulog,1110)
          stop
        end if
c
        idgm=0
        iout=0
        write (luttyo,1150)
        read (luttyi,*) idgm
        if (idtf.ge.1) then
          write (luttyo,1160)
          read (luttyi,*) iout
        end if 
c
c----------------------------------------------------------------------
c     write input information in log file
c----------------------------------------------------------------------
c
      call wrfout(lulog)
c
c----------------------------------------------------------------------
c     read the files that contains starting vector, eigenvalues and
c     eigenvectors for pS=1 subspace and for pS=0 subspace if necessary.
c----------------------------------------------------------------------
c
c                    *** read the starting vector : pS=1 ***
c
      if (idgm.ne.1) then
        write (luttyo,2000)
        open (unit=ludisk,file=vtnamx,status='old',
     #       access='sequential',form='unformatted')
        read (ludisk) (stv(i,1),i=1,ndim1)
        close (unit=ludisk)
      end if
c                    *** read the eigenvalues : pS=1 ***
c
      open (unit=ludisk,file=egnamx,status='old',
     #     access='sequential',form='formatted')
      if (idgm.ne.1) then
        read (ludisk,2200) (j,k,egvalx(i),wt,i=1,nev1)
      else
        read (ludisk,2200) (j,k,egvalx(i),otnstv(i),i=1,nev1)
        do 11 i=1,nev1
          otnstv(i)=cdsqrt(otnstv(i))
          otstv(i)=dconjg(otnstv(i))
 11     continue
      end if
      close (unit=ludisk)
c  substract b0 term from the eigenvalues
      do 15 i=1,nev1
 15     egvalx(i)=egvalx(i)+ci*field
c
c     NO NUCLEAR MODULATION
      if (inz.eq.1) then
        write (luttyo,*) 'NO NUCLEAR MODULATION'
        inz=0
      end if
c
      if (inz.eq.1) then
c                    *** read the eigenvalues : pS=-1 ***
        open (unit=ludisk,file=egnamy,status='old',
     #       access='sequential',form='formatted')
        read (ludisk,2200) (j,k,egvaly(i),wt,i=1,nev1)
        close (unit=ludisk)
c  substract b0 term from the eigenvalues
        do 16 i=1,nev1
 16       egvaly(i)=egvaly(i)-ci*field
      else
        do 17 i=1,nev1
 17       egvaly(i)=dconjg(egvalx(i))
      end if
c
c                    *** read the eigenvectors : pS=1 ***
c
      if (idgm.eq.0) then
c
        open (unit=ludisk,file=evnamx,status='old',
     #        access='sequential',form='unformatted')
        do 20 j=1,nev1
 20       read (ludisk) (oomat(i,j),i=1,ndim1)
        close (unit=ludisk)
c
        if (inz.eq.1) then
c                    *** read the eigenvectors : pS=-1 ***
c
          open (unit=ludisk,file=evnamy,status='old',
     #          access='sequential',form='unformatted')
          do 30 j=1,nev1
 30         read (ludisk) (oonmat(i,j),i=1,ndim1)
          close (unit=ludisk)
        else
          do 35 j=1,nev1
          do 35 i=1,ndim1
 35         oonmat(i,j)=dconjg(oomat(i,j))
        end if
c
      end if
c        
      cpu1=dtime(tarray)
      write (lulog,1200) cpu1
c
c---------------------------------------------------------------------
c     Conversion of unit of time constants and the eigenvalues
c---------------------------------------------------------------------
c
      tfix=tfix*mone
      if (idtf.eq.0) then
        init1=init1*mone
        stpt1=stpt1*mone
        freq1=one/stpt1
        inif1=-freq1/two
        res1=freq1/(npt1-1)
        if (iexp.ne.0) then
          init2=init2*mone
          stpt2=stpt2*mone
          freq2=one/stpt2
          inif2=-freq2/two
          if (npt2.ne.1) res2=freq2/(npt2-1)
        end if
      else if (idtf.eq.1) then
        init1=init1*mone
        stpt1=stpt1*mone
        freq1=one/stpt1
        res1=freq1/(npt1-1)
        if (iexp.ne.0) then 
          freq2=finf2-inif2
          if (npt2.ne.1) res2=freq2/(npt2-1)
        end if
      else
        freq1=finf1-inif1
        res1=freq1/(npt1-1)
        if (iexp.ne.0) then
          freq2=finf2-inif2
          if (npt2.ne.1) res2=freq2/(npt2-1)
        end if
      end if
c
      gib0=gib0*cfact*itwopi
c
      do 60 i=1,nev1
        egvalx(i)=(egvalx(i)+lwid)*cfact
        egvaly(i)=(egvaly(i)+lwid)*cfact
        do 65 j=1,nev1
 65       plsmix(j,i)=czero
        plsmix(i,i)=cone
 60   continue
c
      if (idgm.eq.1) go to 803
c
c---------------------------------------------------------------------
c     calculate the projection of each eigenvector on starting vector
c     Note : we observe pS=-1 subspace signal which is equivalent to
c     detect S+ operator. (consistent with the usual NMR convention)
c---------------------------------------------------------------------
c
      write (luttyo,2300)
      write (luttyo,2310)
c
      do 100 i=1,nev1
        sum1=czero
        sum2=czero
        do 110 j=1,ndim1
          sum1=sum1+oomat(j,i)*stv(j,1)
 110      sum2=sum2+oonmat(j,i)*stv(j,1)
        otstv(i)=sum1
 100    otnstv(i)=sum2
c
c---------------------------------------------------------------------
c     calculate constant part in 2-D calculation for SECSY
c---------------------------------------------------------------------
c
      write (luttyo,2320)
c
      call zgemul(oonmat,mxdimo,'t',oomat,mxdimo,'n',
     #            plsmix,mxegv,nev1,ndim1,nev1)
c
c=====================================================================
c     calculate 2-D spectrum in time-domain
c=====================================================================
c
 803  write (luttyo,2500)
c
c---------------------------------------------------------------------
c     step out t1 values
c---------------------------------------------------------------------
c
      if (idtf.ne.2) then
c                                        * time-domain calc.
        do 300 it1=1,npt1
          t1=init1+(it1-1)*stpt1
          if (idgm.ne.1) then
            do 305 i=1,nev1
 305          buf1(i,it1)=cdexp(-egvaly(i)*t1)*otnstv(i)
          else
            do 307 i=1,nev1
 307          buf1(i,it1)=dexp(-2.0d0*dreal(egvaly(i))*t1)*otnstv(i)
          end if
 300    continue
c
      else
c                                        * frequency-domain calc.
        do 310 it1=1,npt1
          f1=inif1+(it1-1)*res1
          do 315 i=1,nev1
            buf1(i,it1)=otnstv(i)/(dreal(egvaly(i))*itwopi-ci*f1)
 315      continue
 310    continue
c
      end if
c
c                        ** overwrite oomat here to save memory usage **
      call zgemul(plsmix,mxegv,'t',buf1,mxegv,'n',
     #            oomat,mxdimo,nev1,nev1,npt1)
c
c---------------------------------------------------------------------
c     step out t2 values
c---------------------------------------------------------------------
c
c               *** t2 independent of t1 : G. MILLHAUSER'S CASE ***
c
      if (idgm.eq.1) then
        if (idtf.eq.2) then
c                                        * frequency-domain calc.
          do 320 i=1,nev1
 320        egvalx(i)=egvalx(i)*itwopi
c				* USES approximation (T2 << GIB) *
          do 330 it2=1,npt2
            f2=inif2+(it2-1)*res2
            do 335 i=1,nev1
 335          buf1(i,it2)=otstv(i)*
     #                    dexp(-(dimag(egvalx(i))+f2)**2/gib0**2)
 330      continue
        else
c                                        * time-domain calc.
          do 340 it2=1,npt2
            t2=init2+(it2-1)*stpt2
            do 345 i=1,nev1
 345          buf1(i,it2)=otstv(i)*cdexp(-egvalx(i)*t2)
 340      continue
c
        end if
c
        call zgemul(oomat,mxdimo,'t',buf1,mxegv,'n',
     #              plsmix,mxegv,npt1,nev1,npt2)
c
      else
c
c               *** Explicit calculation ***
c
        if (idtf.eq.2) then
c                                        * frequency-domain calc.
c				* USES approximation (T2 << GIB) *
          do 680 i=1,nev1
 680        egvalx(i)=egvalx(i)*itwopi
          do 682 it2=1,npt2
            f2=inif2+(it2-1)*res2
            do 684 i=1,nev1
 684          buf1(i,it2)=otstv(i)*
     #                    dexp(-(dimag(egvalx(i))+f2)**2/gib0**2)
 682      continue
c
          call zgemul(oomat,mxdimo,'t',buf1,mxegv,'n',
     #                plsmix,mxegv,npt1,nev1,npt2)
c
        else
c                                        * time-domain calc.
          do 350 it1=1,npt1
            t1=init1+(it1-1)*stpt1
            do 360 it2=1,npt2
              t2=init2+(it2-1)*stpt2
              t2=t2+t1
              do 365 i=1,nev1
 365            buf1(i,it2)=otstv(i)*cdexp(-egvalx(i)*t2)
 360        continue
c
            call zgemv('t',nev1,npt2,cone,buf1,mxegv,oomat(1,it1),
     #                 1,czero,plsmix(it1,1),mxegv)
 350      continue
c
        end if
c
      end if
c
      cpu2=dtime(tarray)
      write (lulog,1210) cpu2
c
c---------------------------------------------------------------------
c     Store time domain signal
c---------------------------------------------------------------------
c
      if (itimed.eq.1) then
        open (unit=ludisk,file=fidnam,status='unknown',
     #        access='sequential',form='unformatted')
        do 400 j=1,npt2
 400      write (ludisk) (plsmix(i,j),i=1,npt1)
        close (unit=ludisk)
      end if
c
c---------------------------------------------------------------------
c     Convolution with Gaussian inhomogeneous broadening
c     This can be achieved by simple multiplication with
c     the integrated Gaussian function in time domain, which is
c     an exponential function (see Malonic Acid paper).
c---------------------------------------------------------------------
c
      if ((gib0.gt.1.0D-13).and.(idtf.eq.0)) then
        cst=two*pi*pi*gib0*gib0
        do 450 i=1,npt1
        do 450 j=1,npt2             
          t2=init2+(j-1)*stpt2
          expnt=cst*t2*t2
          gconv=zero
          if (expnt.le.5.0D1)  gconv=dexp(-expnt)
          plsmix(i,j)=plsmix(i,j)*gconv
 450    continue
      end if
c
c---------------------------------------------------------------------
c      Prepare output spectrum for 2D-plot
c---------------------------------------------------------------------
c
      if (idtf.eq.ispc) then
        do 480 j=1,npt2
        do 480 i=1,npt1
          if (iout.eq.0) then
            spec(i,j)=cdabs(plsmix(i,j))
          else
            spec(i,j)=dreal(plsmix(i,j))
          end if
 480    continue
      else
        if ((ispc.ne.0).and.(iexp.ne.0).and.(npt2.gt.1)) then
c                                **  Fourier transform over t2
          do 500 i=1,npt1
            do 510 j=1,npt2
 510          buf(j)=plsmix(i,j)
            call fft(buf,npt2)
            call switch(buf,npt2)
            do 520 j=1,npt2
 520          plsmix(i,j)=buf(j)
 500      continue
        end if
c
        if (ispc.eq.2) then
c                                 **  Fourier transform over t1
          do 550 j=1,npt2
            do 560 i=1,npt1
 560          buf(i)=plsmix(i,j)
            call fft(buf,npt1)
            call switch(buf,npt1)
            do 570 i=1,npt1
              if (npt2.eq.1) plsmix(i,j)=buf(i)
              if (iout.eq.0) then
                spec(i,j)=cdabs(buf(i))
              else
                spec(i,j)=dreal(buf(i))
              end if
 570        continue
 550      continue
        else
          do 580 j=1,npt2
          do 580 i=1,npt1
            if (iout.eq.0) then
              spec(i,j)=cdabs(plsmix(i,j))
            else
              spec(i,j)=dreal(plsmix(i,j))
            end if
 580      continue
        end if
      end if
c
c---------------------------------------------------------------------
c     Preparation of graph (Frequencies in MHz)
c---------------------------------------------------------------------
c
      if (npt2.eq.1) then
        open (unit=ludisk,file=mgname,status='unknown',
     #        access='sequential',form='formatted')
        do 590 i=1,npt1
          if (ispc.eq.0) then
            t1=init1+(i-1)*stpt1
            write (ludisk,3130) t1,plsmix(i,1)
          else
            f1=inif1+(i-1)*res1
            write (ludisk,3130) f1,spec(i,1),plsmix(i,1)
          end if
 590    continue   
        close (unit=ludisk)
        go to 700
      end if
c
      amin=zero
      amax=zero
c
      do 610 j=1,npt2
      do 610 i=1,npt1
        if( spec(i,j).gt.amax ) amax=spec(i,j)
        if( spec(i,j).lt.amin ) amin=spec(i,j)
 610  continue
c
      write (luttyo,3000)
c
      open (unit=ludisk,file=mgname,status='unknown',
     #        access='sequential',form='formatted')
      write (ludisk,3100) 0
      if (ispc.eq.2) then
        write (ludisk,3110) npt2,1,npt2,xaxisf
        write (ludisk,3110) npt1,1,npt1,yaxisf
        write (ludisk,3120) inif2,res2,inif1,res1,amin,amax
      else if (ispc.eq.1) then
        write (ludisk,3110) npt2,1,npt2,xaxisf
        write (ludisk,3110) npt1,1,npt1,yaxist
        write (ludisk,3120) inif2,res2,init1*1.D3,stpt1*1.D3,
     #                      amin,amax
      else
        write (ludisk,3110) npt2,1,npt2,xaxist
        write (ludisk,3110) npt1,1,npt1,yaxist
        write (ludisk,3120) init2*1.D3,stpt2*1.D3,init1*1.D3,
     #                      stpt1*1.D3,amin,amax
      end if
      write (ludisk,3120) ((spec(i,j),j=1,npt2),i=1,npt1)
c
      close (unit=ludisk)
c
 700  cpu3=dtime(tarray)
      write (lulog,1220) cpu3
c
c---------------------------------------------------------------------
c     print out trailer and close log file
c---------------------------------------------------------------------
c
 999  continue
      write (luttyo,1000)
      write (lulog,1000)
c
c======================================================================
c     end of loop over calculations
c======================================================================
c
      cputot = cputot+cpu1+cpu2+cpu3
c
      write (luttyo,6000) cputot
      write (lulog,6000) cputot
c
      close (unit=lulog)
c
 900  continue
c
      stop
c
c=====================================================================
c     format statements
c=====================================================================

c
 1000 format(/,2x,70('#'),/)
 1010 format(25x,'PROGRAM : FSESE',/,25x,'---------------',/)
 1100 format(/,20x,'*** CANNOT FIND FILE ',a,' ***',/)
 1110 format(20x,'*** INP file not for time-domain calculation ***')
c
 1150 format(2x,'Tr(O*)O=I ? [0=No, 1=Yes] : ')
 1160 format(2x,'Spectrum along f2 is assumed to be Gaussian.',
     #       /,2x,'Output along f1 ? (0=magn, 1=absp) : ')
c
 1200 format(/,2x,'CPU time in reading files : ',f10.2)
 1210 format(/,2x,'CPU time in calculating 2-D spectrum : ',f10.2)
 1220 format(/,2x,'CPU time in processing data : ',f10.2)
c
 2000 format(/,8x,'*** Reading Starting vector, eigenvalues and',
     #        ' eigenvectors ***')
 2200 format(i4,2x,i4,4x,g14.7,1x,g14.7,2x,g14.7,1x,g14.7)
 2300 format(/,18x,'*** Calculating time-independent part ***',/)
 2310 format(/,25x,'evolution & detection period',/)
 2320 format(/,25x,'mixing period',/)
c
 2500 format(/,18x,'*** Calculating 2-D spectrum ***',/)
c
 3000 format(/,18x,'*** Writing 2-D spectrum ***',/)
 3100 format(i10)
 3110 format(3i10,5x,a30)
 3120 format(8e10.4)
 3130 format(g14.7,4x,g14.7,4x,g14.7,2x,g14.7)
c
 5000 format(15x,39('='),/,
     #     15x,'Calculation no. ',I2,' using file ',a,/,
     #     15x,39('='),/)
 6000 format(/,2x,'Total CPU time (seconds) : ',f10.2)
c
c=====================================================================
c
      end
