c*********************************************************************
c
c                       ==============
c                       PROGRAM : MOMD
c                       ==============
c
c This program generates 2D-spectrum for MOMD (Microscopically Ordered,
c Macroscopically Disordered) type system.
c
c It calculates the eigenvalues and eigenvectors for each director tilt
c angles and they are discarded after the spectral calculation specified.
c Therefore the calculation for a single orientation should be fast.
c
c It is strongly recommended to use the pruned basis set, obtained by
c running program EPRBF or COMBAS.  The dimension of the matrix should be
c less than 100 to be effective.
c
c       Includes :
c               momdim.inc
c               stdio.inc
c               eprdat.inc
c               indexf.inc
c               tdmat.inc
c               tdexp.inc
c               timer.inc
c               tdfnam.inc
c
c       Uses :
c               getidt.f
c               tdsnam.f
c               rdfinp.f
c               wrfinp.f
c               wrfout.f
c               cd2km.f
c               evesa.f
c               fft.f
c               switch.f
c
c*********************************************************************
c
      program momd
c
      implicit none
c
      include 'momdim.inc'
      include 'stdio.inc'
      include 'eprdat.inc'
      include 'indexf.inc'
      include 'tdexp.inc'
      include 'tdfnam.inc'
      include 'tdmat.inc'
      include 'timer.inc'
c
      logical fexist    
c
      double precision zero,one,two,mone
      parameter (zero=0.0D0,one=1.0D0,two=2.0D0,mone=1.0D-3)
      double precision pi,twopi,gtf,cfact,itwopi
      parameter (gtf=2.802471512)
      complex*16 czero,cone,ci
      parameter (czero=(0.0D0,0.0D0),cone=(1.0D0,0.0D0),
     #           ci=(0.0D0,1.0D0))
c
      integer i,j,k,l,m,n,ndimd,mxcal,ncalc,it1,it2,ierr,
     #        iort,bdwo,bdwd,ntmp
      double precision t1,t2,f1,f2,freq1,freq2,res1,res2,gib,
     #       cspsi,cst,expnt,gconv,amin,amax,cpuo,cpud,cpuor
      double precision d2km(2,5,5)
      complex*16 cspec,sum1,sum2,wt
c
      complex*16 egvalx(mxegv),egvaly(mxegv),egvalz(mxegv),
     #           x(mxdim),cwsp(mxdim),xspec(mxsp1,mxsp2)
c
      character*30 xaxist,yaxist,xaxisf,yaxisf
      data xaxisf/'f2 (MHz)'/
      data yaxisf/'f1 (MHz)'/
      data xaxist/'t2 (ns)'/
      data yaxist/'t1 (ns)'/
c
c#####################################################################
c
      pi=4.0d0*datan(one)
c
c----------------------------------------------------------------------
c Timing call changed for compatibility with UNIX
c      call cputime(cpu0,ircode)
c----------------------------------------------------------------------
c
      cputot = dtime( tarray )
      twopi=two*pi
      itwopi=one/twopi
      cfact=twopi*gtf
c
      write (luttyo,1000)
      write (luttyo,1010)
c
      mxcal=10
      call getidt(n,mxcal)
c
      do 900 ncalc=1,n
c
c---------------------------------------------------------------------
c       construct file names
c---------------------------------------------------------------------
c
        ioflid = iofile(ncalc)
        dtflid = dtfile(ncalc)
        call tdsnam
c
c----------------------------------------------------------------------
c       open log file and write out header 
c----------------------------------------------------------------------
c
        open (unit=lulog,file=rpname,access='sequential',
     #        form='formatted')
c
        rewind (unit=lulog)
c
        write (luttyo,1000)
        write (luttyo,5000) ncalc,ipname(:namlth)
        write (lulog,5000) ncalc,ipname(:namlth)
c
c----------------------------------------------------------------------
c     read epr parameters from parameter file for off-diagonal
c----------------------------------------------------------------------
c
        inquire(file=prnamx,exist=fexist)
        if (fexist) then
          call rdfdat(prnamx,ierr)
        else
          write (luttyo,1100) prnamx(:nmlth1)
          write (lulog,1100) prnamx(:nmlth1)
          go to 999
        endif
c
c----------------------------------------------------------------------
c       read 2D parameters from input file
c----------------------------------------------------------------------
c
        inquire(file=ipname,exist=fexist)
c
        if (fexist) then
          call rdfinp(ipname)
        else
          write (luttyo,1100) ipname(:namlth)
          write (lulog,1100) ipname(:namlth)
          go to 999
        endif
c
        if ((itimed.ne.0).and.(idtf.ne.0)) then
          write (luttyo,5100)
          write (lulog,5100)
          stop
        end if
c
c----------------------------------------------------------------------
c     write input information in log file
c----------------------------------------------------------------------
c
        call wrfout(lulog)
        call wrfprm(prnamx,namlth,lulog)
c
c---------------------------------------------------------------------
c     Conversion of unit of time constants
c---------------------------------------------------------------------
c
        tfix=tfix*mone
        if (idtf.eq.0) then
          init1=init1*mone
          stpt1=stpt1*mone
          freq1=one/stpt1
          inif1=-freq1/two
          res1=freq1/(npt1-1)
          if (iexp.ne.0) then
            init2=init2*mone
            stpt2=stpt2*mone
            freq2=one/stpt2
            inif2=-freq2/two
            res2=freq2/(npt2-1)
          end if
        else if (idtf.eq.1) then
          init1=init1*mone
          stpt1=stpt1*mone
          freq1=one/stpt1
          res1=freq1/(npt1-1)
          if (iexp.ne.0) then 
            freq2=finf2-inif2
            res2=freq2/(npt2-1)
          end if
        else
          freq1=finf1-inif1
          res1=freq1/(npt1-1)
          if (iexp.ne.0) then
            freq2=finf2-inif2
            res2=freq2/(npt2-1)
          end if
        end if
c
        gib0=gib0*cfact/twopi
        gib2=gib2*cfact/twopi
c
        if (gib2.gt.1.0D-13) then
          call cd2km(d2km,zero,psi,zero)
          gib=gib0+gib2*d2km(1,3,3)
        end if
c
c---------------------------------------------------------------------
c       Starting vector for off-diagonal (orientation-invariant)
c---------------------------------------------------------------------
c
        ips=1
        call fbaso(ixnamx)
        ndimo=ndim
        call fbasd(ixnamx)
        ndimd=ndim
c
        if (ndimd.gt.mxdim) then
          write (luttyo,5200)
          write (lulog,5200)
          go to 999
        end if
c
        write (luttyo,1200) ndimo,ndimd
        write (lulog,1200) ndimo,ndimd
c
        ndim=ndimo
        call stvec(x,cwsp,egvalx,oomat,xdim,0)
c
c======================================================================
c       loop over orientations
c======================================================================
c
        do 910 j=1,npt2
        do 910 i=1,npt1
 910      xspec(i,j)=czero
c
        cpu1 = dtime( tarray )
c
        if (nort.gt.1) then
          write (luttyo,1300)
          write (lulog,1300)
        else
          nort=1
        end if
c
        do 800 iort=1,nort
c
          if (nort.gt.1) then
            cspsi=dfloat(iort-1)/dfloat(nort-1)
            psi=qarcos(qextd(cspsi))*180.0D0/pi
          end if
c
          if (irec.eq.1) write (lulog,1350) iort,psi
c
c---------------------------------------------------------------------
c     calculate eigenvalues & eigenvectors for a given orientation
c---------------------------------------------------------------------
c
          ips=1
          ndim=ndimo
          call evesa(x,egvalx,oomat,bdwo)
          nevo=nev
          nev1=nev
c
          if (irec.eq.1) then
            write (lulog,1360) nev
            write (lulog,1390) (i,egvalx(i),i=1,nev)
          end if
          do 10 i=1,nev1
 10         egvalx(i)=(egvalx(i)+ci*b0)*cfact
c
          if (inz.eq.1) then
            ips=-1
            call evesa(x,egvaly,oonmat,bdwo)
            nevo=min(nev,nevo)
            nev1=min(nev,nev1)
            if (irec.eq.1) then
              write (lulog,1370) nev
              write (lulog,1390) (i,egvaly(i),i=1,nev)
            end if
            do 12 i=1,nev1
 12           egvaly(i)=(egvaly(i)-ci*b0)*cfact
          else
            do 14 i=1,nev1
 14           egvaly(i)=dconjg(egvalx(i))
            do 16 j=1,nev1
            do 16 i=1,ndimo
 16           oonmat(i,j)=dconjg(oomat(i,j))
          end if
c
          cpu2 = dtime( tarray )
          cpuo=cpu2
c
          if (nort.le.1) then
            write (luttyo,1400) neltot,nelv,bdwo,nev1,cpuo
            write (lulog,1400) neltot,nelv,bdwo,nev1,cpuo
          end if 
c
          if (iexp.gt.2) then
            ips=0
            ndim=ndimd
            call stvec(stv,x,egvalx,oomat,xdim,1)
            call evesa(stv,egvalz,odmat,bdwd)
            nev0=nev
            if (irec.eq.1) then
              write (lulog,1380) nev
              write (lulog,1390) (i,egvalz(i),i=1,nev)
            end if
            do 30 i=1,nev0
 30           egvalz(i)=egvalz(i)*cfact
c
            cpu3 = dtime( tarray )
            cpud=cpu3
c
            if (nort.le.1) then
              write (luttyo,1410) neltot,bdwd,nev0,cpud
              write (lulog,1410) neltot,bdwd,nev0,cpud
            end if 
c
          else
            bdwd=0
            nev0=0
            cpud=zero
          end if
c
c---------------------------------------------------------------------
c     calculate the projection of each eigenvector on starting vector
c---------------------------------------------------------------------
c
          if (lwid.gt.1.0D-13) then
            lwid=lwid*cfact
            do 50 i=1,nev1
              egvalx(i)=egvalx(i)+lwid
              egvaly(i)=egvaly(i)+lwid
 50         continue
          end if  
c
          if (icomb.eq.1) then
            do 60 i=1,nev1
              egvaly(i)=egvalx(i)
              do 65 j=1,ndimo
 65             oonmat(j,i)=oomat(j,i)
 60         continue
          end if
c
          do 100 i=1,nev1
            sum1=czero
            sum2=czero
            do 110 j=1,ndimo
              sum1=sum1+oomat(j,i)*x(j)
 110          sum2=sum2+oonmat(j,i)*x(j)
            otstv(i)=sum1
 100        otnstv(i)=sum2
c
c---------------------------------------------------------------------
c     calculate constant part in 2-D calculation for the given
c     experimental type
c---------------------------------------------------------------------
c
c                 *** 1D-FID ***
c
          do 120 j=1,nev1
            do 125 i=1,nev1
 125          plsmix(i,j)=czero
            plsmix(j,j)=cone
 120      continue
c
c                 *** 2 Pulse exp. : COSY or SECSY ***
c
          if ((iexp.eq.1).or.(iexp.eq.2)) then
            call zgemul(oonmat,mxdimo,'t',oomat,mxdimo,'n',
     #                  plsmix,mxegv,nev1,ndimo,nev1)
c
          else if (iexp.ge.3) then
c
            if ((iexp.eq.3).or.(iexp.eq.4)) then
c
c                 *** 3 Pulse exp. : ELDOR, Echo-ELDOR ***
c
              do 190 i=1,nev0
 190            egvalz(i)=cdexp(-egvalz(i)*tfix)
c
              call zgemul(oomat,mxdimo,'t',odmat,mxdimo,'n',
     #                    buf1,mxegv,nev1,ndimo,nev0)
c
c                  ** overwrite oomat here to save memory usage **
c
              call zgemul(oonmat,mxdimo,'t',odmat,mxdimo,'n',
     #                    oomat,mxdimo,nev1,ndimo,nev0)
c
              do 230 i=1,nev1
              do 230 j=1,nev1
                sum1=czero
                do 235 k=1,nev0
 235              sum1=sum1+oomat(i,k)*egvalz(k)*buf1(j,k)
 230            plsmix(i,j)=sum1*icomb
c
            else
c
c                 *** 3 Pulse exp. : Stim.-SECSY ***
c
              do 240 i=1,nev1
 240            egvaly(i)=cdexp(-egvaly(i)*tfix)
c
              do 245 i=1,nev1
 245            buf(i)=otnstv(i)*egvaly(i)
c
              call zgemul(oonmat,mxdimo,'t',odmat,mxdimo,'n',
     #                    buf1,mxegv,nev1,ndimo,nev0)
c
              do 260 i=1,nev0
                sum1=czero
                do 265 j=1,nev1
 265              sum1=sum1+buf1(j,i)*buf(j)
 260            otnstv(i)=sum1*icomb
c
              call zgemul(odmat,mxdimo,'t',oomat,mxdimo,'n',
     #                    plsmix,mxegv,nev0,ndimo,nev1)
c
            end if
c
          end if
c
c=====================================================================
c     calculate 2-D spectrum in time-domain
c=====================================================================
c
c---------------------------------------------------------------------
c     step out t1 values
c---------------------------------------------------------------------
c
          if (idtf.ne.2) then
c                                        * time-domain calc.
            do 300 it1=1,npt1
              t1=init1+(it1-1)*stpt1
              if (iexp.ne.5) then
                do 305 i=1,nev1
 305              buf1(i,it1)=cdexp(-egvaly(i)*t1)*otnstv(i)
              else
                do 307 i=1,nev0
 307              buf1(i,it1)=cdexp(-egvalz(i)*t1)*otnstv(i)
              end if
 300        continue
c
          else
c                                        * frequency-domain calc.
            do 310 it1=1,npt1
              f1=inif1+(it1-1)*res1
              if (iexp.ne.5) then
                do 315 i=1,nev1
 315              buf1(i,it1)=otnstv(i)/(egvaly(i)*itwopi+ci*f1)
              else
                do 317 i=1,nev0
 317              buf1(i,it1)=otnstv(i)/(egvalz(i)*itwopi+ci*f1)
              end if
 310        continue
c
          end if
c
          ntmp=nev1
          if (iexp.eq.5) ntmp=nev0
c                        ** overwrite oomat here to save memory usage **
          call zgemul(plsmix,mxegv,'t',buf1,mxegv,'n',
     #                oomat,mxdimo,nev1,ntmp,npt1)
c
c---------------------------------------------------------------------
c     step out t2 values
c---------------------------------------------------------------------
c
c               *** t2 independent of t1 : no SECSY or echo-ELDOR ***
c
          if (idtf.eq.2) then
c                                        * frequency-domain calc.
            do 320 it2=1,npt2
              f2=inif2+(it2-1)*res2
              do 325 i=1,nev1
 325            buf1(i,it2)=otstv(i)/(egvalx(i)*itwopi+ci*f2)
 320        continue
c
            call zgemul(oomat,mxdimo,'t',buf1,mxegv,'n',
     #                  plsmix,mxegv,npt1,nev1,npt2)
c
          else if ((iexp.eq.1).or.(iexp.eq.3).or.(iexp.eq.5)) then
c
c                                        * time-domain calc.
            do 330 it2=1,npt2
              t2=init2+(it2-1)*stpt2
              if (iexp.eq.5) t2=tfix+t2
              do 335 i=1,nev1
 335            buf1(i,it2)=otstv(i)*cdexp(-egvalx(i)*t2)
 330        continue
c
            call zgemul(oomat,mxdimo,'t',buf1,mxegv,'n',
     #                  plsmix,mxegv,npt1,nev1,npt2)
c
          end if
c
c               *** t2 dependent of t1 : SECSY or echo-ELDOR ***
c
          if ((iexp.eq.2).or.(iexp.eq.4)) then
c
            do 350 it1=1,npt1
              t1=init1+(it1-1)*stpt1
              do 360 it2=1,npt2
                t2=init2+(it2-1)*stpt2
                t2=t2+t1
                do 365 i=1,nev1
 365              buf1(i,it2)=otstv(i)*cdexp(-egvalx(i)*t2)
 360          continue
c
              call zgemv('t',nev1,npt2,cone,buf1,mxegv,oomat(1,it1),
     #                   1,czero,plsmix(it1,1),mxegv)
 350        continue
c
          end if
c
c               *** 1D-FID ***
c
          if (iexp.eq.0) then
            call zgemv('t',nev1,npt1,cone,oomat,mxdimo,otstv,
     #                 1,czero,plsmix(1,1),1)
          end if
c
c                           *** orientation-dependent GIB ***
c
          if (gib2.gt.1.0D-13) then
            cst=two*pi*pi*gib*gib
            do 370 i=1,npt1
              t1=zero
              if ((iexp.eq.0).or.(iexp.eq.1).or.(iexp.eq.3))
     #                          t1=init1+(i-1)*stpt1
              do 380 j=1,npt2             
                t2=init2+(j-1)*stpt2
                if (iexp.eq.0) t2=zero
                expnt=cst*(t2+icomb*t1)**2
                gconv=zero
                if (expnt.le.5.0D1)  gconv=dexp(-expnt)
                plsmix(i,j)=plsmix(i,j)*gconv
 380          continue
 370        continue
c
          end if
c
c                               *** Simpson's integration rule
          cst=cone
          if ((nort.gt.1).and.((iort.eq.1).or.(iort.eq.nort)))
     #                         cst=dcmplx(0.5d0,0.0d0)
          do 390 j=1,npt2
          do 390 i=1,npt1
390         xspec(i,j)=xspec(i,j)+plsmix(i,j)*cst
c
          cpu3 = dtime( tarray )
          cpuor=cpuo+cpud+cpu3
c
          if (nort.gt.1) then
            write (luttyo,1310) iort,psi,bdwo,bdwd,nev1,nev0,cpuo,
     #                          cpud,cpu3,cpuor
            write (lulog,1310) iort,psi,bdwo,bdwd,nev1,nev0,cpuo,
     #                          cpud,cpu3,cpuor
          else
            write (luttyo,1420) cpu3
            write (lulog,1420) cpu3
          end if
c
          cputot=cputot+cpuor
c
 800    continue
c
c=====================================================================
c       end of loop over orientations
c=====================================================================
c
c---------------------------------------------------------------------
c       Store time domain signal
c---------------------------------------------------------------------
c
        if ((itimed.eq.1).or.((nort.gt.1).and.(idtf.eq.0))) then

          open (unit=ludisk,file=fidnam,status='unknown',
     #          access='sequential',form='unformatted')
          do 400 j=1,npt2
 400         write (ludisk) (xspec(i,j),i=1,npt1)
          close (unit=ludisk)
        end if
c
c---------------------------------------------------------------------
c       Convolution with Gaussian inhomogeneous broadening
c---------------------------------------------------------------------
c
        if ((gib0.gt.1.0D-13).and.(gib2.lt.1.0D-13)) then
          cst=two*pi*pi*gib0*gib0
          do 450 i=1,npt1
            t1=zero
            if ((iexp.eq.0).or.(iexp.eq.1).or.(iexp.eq.3))
     #                                    t1=init1+(i-1)*stpt1
            do 460 j=1,npt2             
              t2=init2+(j-1)*stpt2
              expnt=cst*(t2+icomb*t1)**2
              gconv=zero
              if (expnt.le.5.0D1)  gconv=dexp(-expnt)
              xspec(i,j)=xspec(i,j)*gconv
 460        continue
 450      continue
c
        end if
c
c---------------------------------------------------------------------
c       Prepare output spectrum for 2D-plot
c---------------------------------------------------------------------
c
        if (idtf.eq.ispc) then
          do 480 j=1,npt2
          do 480 i=1,npt1
 480        spec(i,j)=cdabs(xspec(i,j))
        else
          if ((ispc.ne.0).and.(iexp.ne.0)) then
c                                 **  Fourier transform over t2
            do 500 i=1,npt1
              do 510 j=1,npt2
 510            buf(j)=xspec(i,j)
              call fft(buf,npt2)
              call switch(buf,npt2)
              do 520 j=1,npt2
 520            xspec(i,j)=buf(j)
 500        continue
          end if
c
          if (ispc.eq.2) then
c                                 **  Fourier transform over t1
            do 550 j=1,npt2
              do 560 i=1,npt1
 560            buf(i)=xspec(i,j)
              call fft(buf,npt1)
              call switch(buf,npt1)
              do 570 i=1,npt1
                if (iexp.eq.0) xspec(i,j)=buf(i)
 570            spec(i,j)=cdabs(buf(i))

 550        continue
          else
            do 580 j=1,npt2
            do 580 i=1,npt1
 580          spec(i,j)=cdabs(xspec(i,j))
          end if
        end if
c
c---------------------------------------------------------------------
c       Preparation of graph (Frequencies in MHz)
c---------------------------------------------------------------------
c
        if (iexp.eq.0) then
          open (unit=ludisk,file=mgname,status='unknown',
     #          access='sequential',form='formatted')
          do 590 i=1,npt1
            if (ispc.eq.0) then
              t1=init1+(i-1)*stpt1
              write (ludisk,3130) t1,xspec(i,1)
            else
              f1=inif1+(i-1)*res1
              write (ludisk,3130) f1,spec(i,1),xspec(i,1)
            end if
 590      continue   
          close (unit=ludisk)
          go to 700
        end if
c
        amin=zero
        amax=zero
c
        do 610 j=1,npt2
        do 610 i=1,npt1
          if( spec(i,j).gt.amax ) amax=spec(i,j)
          if( spec(i,j).lt.amin ) amin=spec(i,j)
 610    continue
c
        open (unit=ludisk,file=mgname,status='unknown',
     #        access='sequential',form='formatted')
        write (ludisk,3100) 0
        if (ispc.eq.2) then
          write (ludisk,3110) npt2,1,npt2,xaxisf
          write (ludisk,3110) npt1,1,npt1,yaxisf
          write (ludisk,3120) inif2,res2,inif1,res1,amin,amax
        else if (ispc.eq.1) then
          write (ludisk,3110) npt2,1,npt2,xaxisf
          write (ludisk,3110) npt1,1,npt1,yaxist
          write (ludisk,3120) inif2,res2,init1*1.D3,stpt1*1.D3,
     #                        amin,amax
        else
          write (ludisk,3110) npt2,1,npt2,xaxist
          write (ludisk,3110) npt1,1,npt1,yaxist
          write (ludisk,3120) init2*1.D3,stpt2*1.D3,init1*1.D3,
     #                        stpt1*1.D3,amin,amax
        end if
        write (ludisk,3120) ((spec(i,j),j=1,npt2),i=1,npt1)
c
        close (unit=ludisk)
c
c---------------------------------------------------------------------
c       print out trailer and close log file
c---------------------------------------------------------------------
c
 700    if (nort.gt.1) then
          write (luttyo,1320)
          write (lulog,1320)
        end if
c
        cpu4 = dtime( tarray )
        write (luttyo,1430) cpu4
        write (lulog,1430) cpu4
c
        cputot=cputot+cpu1+cpu4
c
        write (luttyo,1440) cputot
        write (lulog,1440) cputot
c
        write (luttyo,1000)
        write (lulog,1000)
c
 999	continue
c
        close (unit=lulog)
c
 900  continue
c
      stop
c
c=====================================================================
c     format statements
c=====================================================================

c
 1000 format(/,2x,70('#'),/)
 1010 format(25x,'PROGRAM : MOMD',/,25x,'---------------')
 1100 format(/,20x,'*** CANNOT FIND FILE ',a,' ***',/)
 1200 format(/,2x,'Dimension of pS=1 and pS=0 space : ',i4,2x,i4)
 1300 format(/,2x,70('-'),/,4x,'i',3x,'angle',2x,'bdwo',2x,'bdwd',3x,
     #      'nevo',3x,'nevd',4x,'cpuo',4x,'cpud',4x,'cpus',4x,'cput',
     #      /,2x,70('-'),/)  
 1310 format(2x,i3,3x,f5.2,2x,i3,3x,i3,4x,i3,4x,i3,1x,4(2x,f6.2))
 1320 format(/,2x,70('-'))
 1350 format(/,2x,40('-'),/,2x,'Director tilt angle ',i2,' : psi = ',
     #       g10.3,/,2x,40('-'))
 1360 format(2x,'Eigenvalues of pS = 1 space : nev = ',i4)
 1370 format(2x,'Eigenvalues of pS =-1 space : nev = ',i4)
 1380 format(2x,'Eigenvalues of pS = 0 space : nev = ',i4)
 1390 format(5x,i4,4x,g14.7,1x,g14.7)
 1400 format(/,2x,'Off-diagonal space : ',
     #       /,4x,'Total number of non-zero entries in matrix : ',i6,
     #       /,4x,'Number of non-zero elements in starting vector ',
     #       ': ',i5,/,4x,'Bandwidth = ',i3,5x,'Number of important',
     #       ' eigenvalues = ',i3,/,4x,'CPU time (seconds) : ',f10.2)
 1410 format(/,2x,'Diagonal space : ',
     #       /,4x,'Total number of non-zero entries in matrix : ',i6,
     #       /,4x,'Bandwidth = ',i3,5x,'Number of important',
     #       ' eigenvalues = ',i3,/,4x,'CPU time (seconds) : ',f10.2)
 1420 format(/,2x,'CPU time for spectral calculation (seconds) : ',
     #       f10.2)
 1430 format(/,2x,'CPU time for data processing (seconds) : ',
     #       f10.2)
 1440 format(/,2x,'Total CPU time (seconds) : ',f10.2)
c
 2200 format(4x,i3,4x,g14.7,2x,g14.7)
 3100 format(i10)
 3110 format(3i10,5x,a30)
 3120 format(8e10.4)
 3130 format(g14.7,4x,g14.7,4x,g14.7,2x,g14.7)
c
 5000 format(15x,39('='),/,
     #     15x,'Calculation no. ',I2,' using file ',a,/,
     #     15x,39('='))
 5100 format(10x,'*** INP file not for time-domain calculation ***')
 5200 format(/,20x,'*** MATRIX DIMENSION TOO LARGE ***',/,
     #     4x,'Use program compiled with bigger dimension or reduce',
     #     ' the dimension.')
c
c=====================================================================
c
      end

