c*********************************************************************
c
c                       ================
c                       PROGRAM : PROC2D
c                       ================
c
c This program processes the time domain signal as specified in the 
c input file.
c
c It reads the single quadrature signal (t1,t2) in the file <id>.fid,
c generated by EPRPS or MOMD program with -t flag.  The output data
c is written in <id>.mag file.
c
c Things that can be done :
c
c       INP file :     1. output type [ispc] ; time or frequency
c                      2. inhomogeneous broadening width [gib0]
c       Interactive :  3. spectral type ; real, imaginary, or magnitude
c                      4. frequency range for 2D-plot
c                      5. resolution via zero-filling
c
c       Inclues :
c               stdio.inc
c               tdexp.inc
c               tdfnam.inc
c               rndoff.inc
c
c       Uses :
c               prmpt.f
c               getidt.f
c               tdsnam.f
c               rdfinp.f
c               fft.f
c               switch.f
c
c*********************************************************************
c
      program proc2d
c
      implicit none
c
      include 'stdio.inc'
      include 'tdexp.inc'
      include 'tdfnam.inc'
      include 'rndoff.inc'
c
      logical fprmpt,fexist,fli,flf
      character*80 line
c
      double precision zero,one,two,mone
      parameter (zero=0.0D0,one=1.0D0,two=2.0D0,mone=1.0D-3)
      double precision pi,twopi,gtf,cfact
      parameter (pi=3.1415926535897932384D0,gtf=2.802471512)
      complex*16 czero,ci
      parameter (czero=(0.0D0,0.0D0),ci=(0.0D0,1.0D0))
c
      integer i,j,k,n,mxcal,ierr,ncalc,isig,npt1n,npt1o,nf1i,nf1f,
     #        npt2n,npt2o,nf2i,nf2f,izrf,itoken,concs,f2tax,kt1
      double precision t1,t2,f1,f2,freq1,freq2,res1,res2,cst,
     #        expnt,gconv,amin,amax,oif1,off1,oif2,off2,
     #        nif1,nif2,spec(512,512),ftoken
      complex*16 xspec(512,512),buf(512)
c
      character*30 xaxist,yaxist,xaxisf,yaxisf
      data xaxisf/'f2 (MHz)'/
      data yaxisf/'f1 (MHz)'/
      data xaxist/'t2 (ns)'/
      data yaxist/'t1 (ns)'/
c
      logical prmpt
      external prmpt,itoken,ftoken
c
c#####################################################################
c
      twopi=two*pi
      cfact=twopi*gtf
c
      write (luttyo,1000)
      write (luttyo,1010)
c
      mxcal=10
      call getidt(n,mxcal)
c
      do 900 ncalc=1,n
c
c---------------------------------------------------------------------
c     construct file names
c---------------------------------------------------------------------
c
        ioflid = iofile(ncalc)
        dtflid = dtfile(ncalc)
        call tdsnam
c
c----------------------------------------------------------------------
c     read parameters from input file
c----------------------------------------------------------------------
c
        inquire(file=ipname,exist=fexist)
        if (fexist) then
          call rdfinp(ipname)
        else
          write (luttyo,1100) ipname(:namlth)
          go to 999
        endif
c
        inquire(file=fidnam,exist=fexist)
        if (fexist) then
          do 10 i=1,512
          do 10 j=1,512
 10         xspec(j,i)=czero  
          open (unit=ludisk,file=fidnam,status='old',
     #          access='sequential',form='unformatted')
          do 20 j=1,npt2
 20         read (ludisk) (xspec(i,j),i=1,npt1)
          close (unit=ludisk)
        else
          write (luttyo,1100) fidnam(:namlth)
          go to 999
        end if
c
        if (idtf.ne.0) write (luttyo,1200)
c
c---------------------------------------------------------------------
c     Conversion of unit of time constants
c---------------------------------------------------------------------
c
        tfix=tfix*mone
        init1=init1*mone
        stpt1=stpt1*mone
        init2=init2*mone
        stpt2=stpt2*mone
        freq1=one/stpt1
        freq2=one/stpt2
        inif1=-freq1/two
        inif2=-freq2/two
c
        gib0=gib0*cfact/twopi
c
        concs=0
        if (((iexp.eq.1).or.(iexp.eq.3)).and.(icomb.eq.-1)) then
c want to convert COSY into SECSY format?
 30       write (luttyo,1020) concs
          fprmpt=prmpt(line)
          if (fprmpt) concs=itoken( line )
          if ((concs.lt.0).or.(concs.gt.1)) go to 30
c
          if (concs.eq.1) then
c if f2tax=2, the data for t1>t2 is discarded.
c if f2tax=1, the data for t1<t2 is discarded.
            f2tax=2
 35         write (luttyo,1025) f2tax
            fprmpt=prmpt(line)
            if (fprmpt) f2tax=itoken( line )
            if ((f2tax.lt.1).or.(f2tax.gt.2)) go to 35
          end if
        end if
c
        if (ncalc.eq.1) then
c select output (magnitude, real, imaginary?)
          isig=0
 40       write (luttyo,1030) isig
          fprmpt=prmpt(line)
          if (fprmpt) isig=itoken( line )
          if ((isig.lt.0).or.(isig.gt.2)) go to 40
c get suitable frequency range
          if (ispc.eq.1) then
            oif2=inif2
            off2=-inif2
            write (luttyo,1040) oif2,off2
            fprmpt=prmpt(line)
            if (fprmpt) then
              oif2=ftoken( line )
              off2=ftoken( line )
            end if
          else if (ispc.eq.2) then
            oif1=inif1
            off1=-inif1
            oif2=inif2
            off2=-inif2
            write (luttyo,1050) oif1,off1,oif2,off2
            fprmpt=prmpt(line)
            if (fprmpt) then
              oif1=ftoken( line )
              off1=ftoken( line )
              oif2=ftoken( line )
              off2=ftoken( line )
            end if
          end if
c zero-fill to increase resolution
c izrf determines the factor in the power of 2
          if ((ispc.ge.1).and.(npt1.le.256).and.(npt2.le.256)) then
            izrf=1
 50         write (luttyo,1060) izrf
            fprmpt=prmpt(line)
            if (fprmpt) izrf=itoken( line )
            if (izrf.le.0) then
              write (luttyo,1070)
              izrf=1
            end if
            if ((izrf*npt1.gt.512).or.(izrf*npt2.gt.512)) then
              write (luttyo,1080)
              go to 50
            end if
            if ((izrf.ne.1).and.(izrf.ne.2).and.(izrf.ne.4).and.
     #          (izrf.ne.8).and.(izrf.ne.16)) then
              write (luttyo,1090)
              go to 50
            end if
            npt1n=npt1*izrf
            npt2n=npt2*izrf
          else
            npt1n=npt1
            npt2n=npt2
          end if
c
        end if
c
        res1=freq1/(npt1n-1)
        res2=freq2/(npt2n-1)
c
c---------------------------------------------------------------------
c       Convolution with Gaussian inhomogeneous broadening
c---------------------------------------------------------------------
c
        if (gib0.gt.1.0D-13) then
          cst=two*pi*pi*gib0*gib0
          do 70 i=1,npt1
            if ((iexp.eq.2).or.(iexp.eq.4).or.(iexp.eq.5)) then
              t1=zero
            else
              t1=init1+(i-1)*stpt1
            end if
c
            do 80 j=1,npt2
              t2=init2+(j-1)*stpt2
              expnt=cst*(t2+icomb*t1)**2
              gconv=zero
              if (expnt.le.5.0D1)      gconv=dexp(-expnt)
c
              xspec(i,j)=xspec(i,j)*gconv
 80         continue
 70       continue
c
c---------------------------------------------------------------------
c       Store GIB-broadened spectrum in .fid format
c---------------------------------------------------------------------
c 
          open (unit=ludisk,file=gibnam,status='unknown',
     #          access='sequential',form='unformatted')
          do 90 j=1,npt2
 90         write (ludisk) (xspec(i,j),i=1,npt1)
          close (unit=ludisk)
        end if
c
c---------------------------------------------------------------------
c       Transform the spectrum from COSY Sc- format to SECSY format
c---------------------------------------------------------------------
c 
        if (concs.eq.1) then 
          if (f2tax.eq.2) then
c throw away the data with t2<t1
            kt1=0  
            do 100 i=1,npt1
              t1=init1+(i-1)*stpt1
c skip the region with asymmetric dead time
              if (kt1.eq.0) then
                if ((t1.ge.init2).or.((init2-t1).lt.rndoff)) then
                  kt1=i
                else
                  go to 100
                end if
              end if
c get the threshold index when t2 > t1 starts
              do 102 j=1,npt2
                t2=init2+(j-1)*stpt2
                if ((t2.ge.t1).or.((t1-t2).lt.rndoff)) go to 103
 102          continue
c copy FID and zero-fill
 103          do 104 k=j,npt2
 104            xspec(i-kt1+1,k-j+1)=xspec(i,k)
              do 106 k=npt2-j+2,npt2
 106            xspec(i-kt1+1,k)=czero
 100        continue
c zero out the difference of the dead time in t1 & t2
            do 108 i=npt1-kt1+2,npt1
            do 108 k=1,npt1
 108          xspec(i,k)=czero
c
          else
c apply the same procedure for t1>t2 region
            kt1=0  
            do 110 i=1,npt2
              t2=init2+(i-1)*stpt2
              if (kt1.eq.0) then
                if ((t2.ge.init1).or.((init1-t2).lt.rndoff)) then
                  kt1=i
                else
                  go to 110
                end if

             end if
              do 112 j=1,npt1
                t1=init1+(j-1)*stpt1
                if ((t1.ge.t2).or.((t2-t1).lt.rndoff)) go to 113
 112          continue
 113          do 114 k=j,npt1
 114            xspec(k-j+1,i-kt1+1)=xspec(k,i)
              do 116 k=npt1-j+2,npt1
 116            xspec(k,i-kt1+1)=czero
 110        continue
            do 118 i=npt2-kt1+2,npt2
            do 118 k=1,npt1
 118          xspec(k,i)=czero
          end if    
        end if    
c
c---------------------------------------------------------------------
c       Prepare output spectrum for 2D-plot
c---------------------------------------------------------------------
c
        if (idtf.ne.ispc) then
          if (ispc.ne.0) then
c                                 **  Fourier transform over t2
            do 130 i=1,npt1
              do 140 j=1,npt2n
 140            buf(j)=xspec(i,j)
              call fft(buf,npt2n)
              call switch(buf,npt2n)
              do 150 j=1,npt2n
 150            xspec(i,j)=buf(j)
 130        continue
          end if
c
          if (ispc.eq.2) then
c                                 **  Fourier transform over t1
            do 160 j=1,npt2n
              do 170 i=1,npt1n
 170            buf(i)=xspec(i,j)
              call fft(buf,npt1n)
              call switch(buf,npt1n)
              do 180 i=1,npt1n
 180            xspec(i,j)=buf(i)
 160        continue
          end if
c
        end if
c
        do 200 j=1,npt2n
        do 200 i=1,npt1n
          if (isig.eq.0) then
            spec(i,j)=cdabs(xspec(i,j))
          else if (isig.eq.1) then
            spec(i,j)=dreal(xspec(i,j))
          else
            spec(i,j)=dimag(xspec(i,j))
          end if
 200    continue  
c
c---------------------------------------------------------------------
c       Search range for partial spectrum in frequency domain
c---------------------------------------------------------------------
c
        if ( (ispc.ge.1).and.( ((oif2-inif2).gt.rndoff).or.
     #       ((off2+inif2).gt.rndoff) ) ) then
c obtain the array index for display range for f2
          fli=.true.
          flf=.true.
          do 220 i=1,npt2n
            f2=inif2+(i-1)*res2
            if (fli.and.(f2.gt.oif2)) then
              nf2i=i
              nif2=f2
              fli=.false.
            end if
            if (flf.and.(f2.gt.off2)) then
              nf2f=i-1
              flf=.false.
            end if
 220      continue
          if (fli) then
            nif2=inif2
            nf2i=1
          end if
          if (flf) nf2f=npt2n
          npt2o=nf2f-nf2i+1
        else
          nif2=inif2
          nf2i=1
          nf2f=npt2n
          npt2o=npt2n
        end if
c
        if ( (ispc.eq.2).and.( ((oif1-inif1).gt.rndoff).or.
     #       ((off1+inif1).gt.rndoff) ) ) then
c obtain the array index for display range for f1
          fli=.true.
          flf=.true.
          do 230 i=1,npt1n
            f1=inif1+(i-1)*res1
            if (fli.and.(f1.gt.oif1)) then
              nf1i=i
              nif1=f1
              fli=.false.
            end if
            if (flf.and.(f1.gt.off1)) then
              nf1f=i-1
              flf=.false.
            end if
 230      continue
          if (fli) then
            nif1=inif1
            nf1i=1
          end if
          if (flf) nf1f=npt1n
          npt1o=nf1f-nf1i+1
        else
          nif1=inif1
          nf1i=1
          nf1f=npt1n
          npt1o=npt1n
        end if
c
c---------------------------------------------------------------------
c       Preparation of graph (Frequencies in MHz)
c---------------------------------------------------------------------
c
        amin=zero
        amax=zero
c
        do 410 j=nf2i,nf2f
        do 410 i=nf1i,nf1f
          if( spec(i,j).gt.amax ) amax=spec(i,j)
          if( spec(i,j).lt.amin ) amin=spec(i,j)
 410    continue
c
        open (unit=ludisk,file=mgname,status='unknown',
     #        access='sequential',form='formatted')
c
        write (ludisk,3100) 0
        if (ispc.eq.2) then
          write (ludisk,3110) npt2o,1,npt2o,xaxisf
          write (ludisk,3110) npt1o,1,npt1o,yaxisf
          write (ludisk,3120) nif2,res2,nif1,res1,amin,amax
        else if (ispc.eq.1) then
          write (ludisk,3110) npt2o,1,npt2o,xaxisf
          write (ludisk,3110) npt1o,1,npt1o,yaxist
          write (ludisk,3120) nif2,res2,init1*1.D3,stpt1*1.D3,
     #                        amin,amax
        else
          write (ludisk,3110) npt2o,1,npt2o,xaxist
          write (ludisk,3110) npt1o,1,npt1o,yaxist
          write (ludisk,3120) init2*1.D3,stpt2*1.D3,init1*1.D3,
     #                        stpt1*1.D3,amin,amax
        end if
        write (ludisk,3120) ((spec(i,j),j=nf2i,nf2f),i=nf1i,nf1f)
c
        close (unit=ludisk)
c
 900  continue
c
 999  stop
c
c=====================================================================
c     format statements
c=====================================================================
c
 1000 format(/,2x,70('#'),/)
 1010 format(25x,'PROGRAM : PROC2D',/,25x,'---------------')
 1020 format(2x,'Change COSY into SECSY format? (Y=1,N=0) :  ',i2)
 1025 format(2x,'Which axis does cw spectrum appear along?',
     #       ' (f1=1,f2=2) :  ',i2)
 1030 format(2x,'Spectral format? ( 0=Magnitude, 1=Real, 2=Imag. )',
     #       ' :  ',i1)
 1040 format(2x,'Freq. range for f2 : ',g10.4,2x,g10.4)
 1050 format(2x,'Freq. range for f1 & f2 : ',g10.4,3(2x,g10.4))
 1060 format(2x,'Enter freq. resolution increase (in power of 2)',
     #       ' : ',i2)
 1070 format(2x,'*** No zero filling assumed. ***')
 1080 format(2x,'*** Current dimension is too big for zero-filling ',
     #       '***')
 1090 format(2x,'*** Only power of 2 is valid for FFT. ***') 
 1100 format(/,20x,'*** CANNOT FIND FILE ',a,' ***',/)
 1200 format(/,20x,'*** idtf assumed to be zero. ***')
c
 3100 format(i10)
 3110 format(3i10,5x,a30)
 3120 format(8e10.4)
c
c=====================================================================
c
      end
