c**********************************************************************
c       
c                       ===================
c                       SUBROUTINE : WRFINP
c                       ===================
c
c       NOTE: This is the EPRTD version of subroutine WRFDAT.
c
c       This writes the parameters contained in the common block
c       defined in tdexp.inc into a file in a form that can be read 
c       by rdfdat.f. 
c
c       Notes:
c               Care should be taken to ensure that the common 
c               block definition in the include file tdexp.inc
c               is consistent with the information stored in the
c               disk files by this routine to avoid corrupting 
c               the parameter file.
c
c       Includes:
c               stdio.inc
c               tdexp.inc
c
c**********************************************************************
c
      subroutine wrfinp(ipname,lth)
c
      include 'stdio.inc'
      include 'tdexp.inc'
c
      character*30 ipname
      integer lth
c
c######################################################################
c
c----------------------------------------------------------------------
c     open parameter file
c----------------------------------------------------------------------
c
      open (unit=ludisk,file=ipname,access='sequential',
     #     form='formatted')
c
      write(ludisk,1000) ipname(:lth)
c
      write (ludisk,2000) iexp
      write (ludisk,2005) icomb
      write (ludisk,2010) npt1,npt2
      write (ludisk,2020) idtf,ispc
      if (idtf.eq.0) then
          write (ludisk,2030) int(init1),int(stpt1),
     #                        int(init2),int(stpt2)
      else if (idtf.eq.1) then
          write (ludisk,2035) int(init1),int(stpt1),inif2,finf2
      else
          write (ludisk,2040) inif1,finf1,inif2,finf2
      end if
      if (iexp.gt.2) write (ludisk,2050) tfix
      write (ludisk,2060) ndim1
      if (iexp.lt.3) then
          write (ludisk,2070) nev1
      else
          write (ludisk,2075) nev1,nev0
      end if
      write (ludisk,2080) inz
      write (ludisk,2090) field
c
      write (ludisk,2110) gib0,gib2
      write (ludisk,2200) lwid
c
c----------------------------------------------------------------------
c     close parameter file
c----------------------------------------------------------------------
c
      close (unit=ludisk)
c
      return
c
c======================================================================
c     format statements
c======================================================================
c
 1000 format(/,2x,70('#'),/,28x,'file : ',a,/,
     #     28x,15('-'),/,2x,70('#'),/)
 2000 format(i1,t26,';[iexp] exp. type ',/,t27,
     #     '(1-COSY,2-SECSY,3-ELDOR,4-E.ELDOR,5-S.SECSY)')
 2005 format(i2,t26,';[icomb] Sc+ (1) or Sc- (-1) combination?')
 2010 format(i3,2x,i3,t26,';[npt1,npt2] # of data pts in (t1,t2)')
 2020 format(i1,2x,i1,t26,';[idtf,ispc] calculation & output type')
 2030 format(i5,3(1x,i5),t26,';[init1,stpt1,init2,stpt2] (ns)',
     #     ' ini. and st. in t1, t2')
 2035 format(i2,2x,i2,g9.3,1x,g9.3,t26,';[init1,stpt1,inif2,',
     #     'finf2] (ns,MHz)  : (t1,f2)')
 2040 format(g9.3,3(1x,g9.3),t40,';[inif1,finf1,inif2,finf2] (MHz)',
     #     ' ini. and final in f1, f2')
 2050 format(g13.6,t26,';[tfix] (ns) fixed time interval')
 2060 format(i5,t26,';[ndim1] dimension of SLE matrix',
     #     ' in pS=1 space')
 2070 format(i5,t26,';[nev1] # of sig. egv in off-diag. space')
 2075 format(i5,2x,i5,t26,';[nev1,nev0] # of sig. egv in',
     #     ' off-diag. & diag. space')
 2080 format(i1,t26,';[inz] include nuclear Zeeman? (Y=1/N=0)')
 2090 format(f10.2,t26,';[field] magnetic field')
c
 2110 format(g10.2,2x,g10.2,t26,';[gib0,gib2] (gauss) Gaussian',
     #     ' inhomo. broad')
 2200 format(g10.2,t26,';[lwid] (gauss) Homogeneous line-width')
c
      end
